from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.connection.subscribe_client import SubscribeClient
from notecoin.huobi.connection.websocket_req_client import *
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.account import *
from notecoin.huobi.utils import *
from notecoin.huobi.utils.json_parser import default_parse_data_as_long


class GetAccountAssetValuationService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/account/asset-valuation"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, AccountAssetValuationResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountBalanceBySubUidService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        sub_uid = self.params["sub-uid"]

        def get_channel():
            path = "/v1/account/accounts/{}"
            return path.format(sub_uid)

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return AccountBalance.json_parse_list(data_list)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)


class GetAccountHistoryService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/account/history"

        def parse(dict_data):
            response = dict()
            data_list = dict_data.get("data", [])
            response['data'] = default_parse_list_dict(data_list, AccountHistory, [])
            response['next_id'] = dict_data.get("next-id", 0)
            return response

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountLedgerService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/account/ledger"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, AccountLedger, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountPointService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/point/account"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, AccountPointResult, {})

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountsService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/account/accounts"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, Account, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAggregateSubUserBalanceService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/subuser/aggregate-balance"

        def parse(dict_data):
            data = dict_data.get("data", [])
            return default_parse_list_dict(data, Balance)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetBalanceService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        account_id = self.params["account-id"]

        def get_channel():
            path = "/v1/account/accounts/{}/balance"
            return path.format(account_id)

        def parse(dict_data):
            data = dict_data.get("data", {})
            balance_list = data.get("list", [])
            return default_parse_list_dict(balance_list, Balance, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)

    def get_request(self, **kwargs):
        account_id = self.params["account-id"]

        def get_channel():
            path = "/v1/account/accounts/{}/balance"
            return path.format(account_id)

        def parse(dict_data):
            data = dict_data.get("data", {})
            balance_list = data.get("list", [])
            return default_parse_list_dict(balance_list, Balance, [])

        return RestApiSyncClient(**kwargs).create_request(HttpMethod.GET_SIGN, get_channel(), self.params, parse)


class PostAccountTransferService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/account/transfer"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, AccountTransferResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostTransferBetweenFuturesAndProService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/futures/transfer"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostPointTransferService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/point/transfer"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, AccountPointTransferResult, {})

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSubUidManagementService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/management"

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), SubUidManagement)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSubaccountTransferService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/subuser/transfer"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class ReqAccountBalanceService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        client_req_id = self.params["client_req_id"]

        def subscription(connection):
            connection.send(request_account_list_channel(client_req_id))

        def parse(dict_data):
            req_obj = AccountBalanceReq()
            req_obj.ts = dict_data.get("ts", 0)
            req_obj.topic = dict_data.get("topic", 0)
            req_obj.cid = dict_data.get("cid", 0)
            data_list = dict_data.get("data", [])
            req_obj.data = AccountBalance.json_parse_list(data_list)
            return req_obj

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                                          parse,
                                                          callback,
                                                          error_handler,
                                                          is_trade=True)


class SubAccountUpdateV2Service:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        mode = self.params["mode"]

        def subscription(connection):
            connection.send(accounts_update_channel(mode))

        def parse(dict_data):
            account_change_event = AccountUpdateEvent()
            account_change_event.ch = dict_data.get("ch")
            data = dict_data.get("data", {})
            if data and len(data):
                account_change_event.data = default_parse_list_dict(data, AccountUpdate)

            return account_change_event

        SubscribeClient(**kwargs).execute_subscribe_v2(subscription,
                                                       parse,
                                                       callback,
                                                       error_handler,
                                                       is_trade=True)



import time

from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.connection.subscribe_client import SubscribeClient
from notecoin.huobi.connection.websocket_req_client import *
from notecoin.huobi.constant import *
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.trade import *
from notecoin.huobi.model.trade.batch_create_order import BatchCreateOrder
from notecoin.huobi.utils import *
from notecoin.huobi.utils.json_parser import default_parse_data_as_long


class GetFeeRateService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/fee/fee-rate/get"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, FeeRate, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetHistoryOrdersService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/history"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return Order.json_parse_list(data_list)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetMatchResultsByOrderIdService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        order_id = self.params["order_id"]

        def get_channel():
            path = "/v1/order/orders/{}/matchresults"
            return path.format(order_id)

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, MatchResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)


class GetMatchResultsService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/matchresults"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, MatchResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetOpenOrdersService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/openOrders"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return Order.json_parse_list(data_list)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetOrderByClientOrderIdService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders/getClientOrder"

        def parse(dict_data):
            data_dict = dict_data.get("data", {})
            return Order.json_parse(data_dict)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetOrderByIdService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        order_id = self.params["order_id"]

        def get_channel():
            path = "/v1/order/orders/{}"
            return path.format(order_id)

        def parse(dict_data):
            data_dict = dict_data.get("data")
            return Order.json_parse(data_dict)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)


class GetOrdersService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return Order.json_parse_list(data_list)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetTransactFeeRateService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/reference/transact-fee-rate"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, TransactFeeRate, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class PostBatchCancelOpenOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders/batchCancelOpenOrders"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse(data, BatchCancelCount)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostBatchCancelOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders/batchcancel"

        def parse(dict_data):
            data = dict_data.get("data", {})
            return default_parse_fill_directly(data, BatchCancelResult)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostBatchCreateOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/batch-orders"

        def parse(dict_data):
            data = dict_data.get("data", [])
            return default_parse_list_dict(data, BatchCreateOrder, [])

        return RestApiSyncClient(**kwargs).request_process_post_batch(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostCancelClientOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders/submitCancelClientOrder"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostCancelOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        order_id = self.params["order_id"]

        def get_channel():
            path = "/v1/order/orders/{}/submitcancel"
            return path.format(order_id)

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)
        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, get_channel(), self.params, parse)


class PostCreateOrderService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/orders/place"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostTransferFuturesProService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/futures/transfer"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class ReqOrderDetailService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        order_id = self.params["order-id"]
        client_req_id = self.params["cid"]

        def subscription(connection):
            connection.send(request_order_detail_channel(order_id, client_req_id))

        def parse(dict_data):
            order_update_event = default_parse(dict_data, OrderDetailReq, OrderListItem)

            return order_update_event

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                                          parse,
                                                          callback,
                                                          error_handler,
                                                          is_trade=True)


class ReqOrderListService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol = self.params["symbol"]
        account_id = self.params["account-id"]
        order_states = self.params["states"]
        client_req_id = self.params["client-req-id"]

        def subscription(connection):
            connection.send(request_order_list_channel(symbol=symbol, account_id=account_id,
                            states_str=order_states, client_req_id=client_req_id, more_key=self.params))

        def parse(dict_data):
            order_update_event = default_parse(dict_data, OrderListReq, OrderListItem)

            return order_update_event

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                                          parse,
                                                          callback,
                                                          error_handler,
                                                          is_trade=True)


class SubOrderUpdateV2Service:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]

        def subscription(connection):
            for val in symbol_list:
                connection.send(orders_update_channel(val))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, OrderUpdateEvent, OrderUpdate)

        SubscribeClient(**kwargs).execute_subscribe_v2(subscription,
                                                       parse,
                                                       callback,
                                                       error_handler,
                                                       is_trade=True)


class SubTradeClearingV2Service:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(trade_clearing_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return TradeClearingEvent.json_parse(dict_data)

        SubscribeClient(**kwargs).execute_subscribe_v2(subscription,
                                                       parse,
                                                       callback,
                                                       error_handler,
                                                       is_trade=True)




from notecoin.huobi.constant import *


class ChainDepositAddress:
    """
    The deposit address.

    :member
        currency: The crypto currency to deposit.
        address: Deposit address
        addressTag: Deposit address tag.
        chain: Block chain name.
    """

    def __init__(self):
        self.currency = ""
        self.address = ""
        self.addressTag = ""
        self.chain = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.addressTag, format_data + "addressTag")
        PrintBasic.print_basic(self.chain, format_data + "Chain")


class ChainWithdrawAddress:
    """
    The deposit address.

    :member
        currency: The crypto currency to deposit.
        address: Deposit address
        addressTag: Deposit address tag.
        chain: Block chain name.
    """

    def __init__(self):
        self.currency = ""
        self.address = ""
        self.addressTag = ""
        self.chain = ""
        self.note = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.addressTag, format_data + "addressTag")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.note, format_data + "Note")


class DepositHistoryItem:
    """
     The deposit history

     :member
         id: The transfer id.
         currency: The crypto currency to deposit.
         txHash: The on-chain transaction hash.
         amount: The number of crypto asset transferred in its minimum unit.
         address: The deposit source address.
         addressTag: The user defined address tag.
         deposit_state: The deposit state of this transfer.
         created_timestamp: The UNIX formatted timestamp in UTC for the transfer creation.
         updated_timestamp: The UNIX formatted timestamp in UTC for the transfer's latest update.
     """

    def __init__(self):
        self.id = 0
        self.currency = ""
        self.txHash = ""
        self.chain = ""
        self.amount = 0.0
        self.address = ""
        self.addressTag = ""
        self.deposit_state = WithdrawState.INVALID
        self.created_timestamp = 0
        self.updated_timestamp = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.txHash, format_data + "Trade Hash")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.addressTag, format_data + "Address Tag")
        PrintBasic.print_basic(self.deposit_state, format_data + "Deposit State")
        PrintBasic.print_basic(self.created_timestamp, format_data + "Create Time")
        PrintBasic.print_basic(self.updated_timestamp, format_data + "Update Time")


class DepositHistory:
    """
    The deposit history

    :member
        nextId: next id.
        data: history list.
    """

    def __init__(self):
        self.data = list()
        self.nextId = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.nextId, format_data + "NextId")
        if self.data and len(self.data):
            for item in self.data:
                item.print_object()
                PrintBasic.print_basic("", format_data + "")


class Deposit:
    """
    The latest status for deposits

    :member
        id: The transfer id.
        currency: The crypto currency to deposit.
        tx_hash: The on-chain transaction hash.
        amount: The number of crypto asset transferred in its minimum unit.
        address: The deposit source address.
        address_tag: The user defined address tag.
        fee: The amount of fee taken by Huobi in this crypto's minimum unit.
        created_at: The UNIX formatted timestamp in UTC for the transfer creation.
        updated_at: The UNIX formatted timestamp in UTC for the transfer's latest update.
        state: The deposit state of this transfer.
    """

    def __init__(self):
        self.id = 0
        self.type = DepositWithdraw.DEPOSIT
        self.currency = ""
        self.tx_hash = ""
        self.amount = 0.0
        self.chain = ""
        self.address = ""
        self.address_tag = ""
        self.fee = 0.0
        self.created_at = 0
        self.updated_at = 0
        self.state = DepositState.INVALID

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Operate Type")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.tx_hash, format_data + "Trade Hash")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.address_tag, format_data + "Address Tag")
        PrintBasic.print_basic(self.fee, format_data + "Fee")
        PrintBasic.print_basic(self.state, format_data + "Deposit State")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.updated_at, format_data + "Update Time")


class WithdrawQuota:
    """
    Withdraw Quota info.

    :member
        chain: Block chain name.
        maxWithdrawAmt: Maximum withdraw amount in each request.
        withdrawQuotaPerDay: Maximum withdraw amount in a day
        remainWithdrawQuotaPerDay: Remaining withdraw quota in the day
        withdrawQuotaPerYear: Maximum withdraw amount in a year
        remainWithdrawQuotaPerYear: Remaining withdraw quota in the year
        withdrawQuotaTotal: Maximum withdraw amount in total
        remainWithdrawQuotaTotal: Remaining withdraw quota in total
    """

    def __init__(self):
        self.chain = ""
        self.maxWithdrawAmt = ""
        self.withdrawQuotaPerDay = ""
        self.remainWithdrawQuotaPerDay = ""
        self.withdrawQuotaPerYear = ""
        self.remainWithdrawQuotaPerYear = ""
        self.withdrawQuotaTotal = ""
        self.remainWithdrawQuotaTotal = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.maxWithdrawAmt, format_data + "maxWithdrawAmt")
        PrintBasic.print_basic(self.withdrawQuotaPerDay, format_data + "withdrawQuotaPerDay")
        PrintBasic.print_basic(self.remainWithdrawQuotaPerDay, format_data + "remainWithdrawQuotaPerDay")
        PrintBasic.print_basic(self.withdrawQuotaPerYear, format_data + "withdrawQuotaPerYear")
        PrintBasic.print_basic(self.remainWithdrawQuotaPerYear, format_data + "remainWithdrawQuotaPerYear")
        PrintBasic.print_basic(self.withdrawQuotaTotal, format_data + "withdrawQuotaTotal")
        PrintBasic.print_basic(self.remainWithdrawQuotaTotal, format_data + "remainWithdrawQuotaTotal")


class Withdraw:
    """
    The latest status for withdraws.

    :member
        id: The transfer id.
        currency: The crypto currency to deposit.
        tx_hash: The on-chain transaction hash.
        amount: The number of crypto asset transferred in its minimum unit.
        address: The deposit source address.
        address_tag: The user defined address tag.
        fee: The amount of fee taken by Huobi in this crypto's minimum unit.
        created_at: The UNIX formatted timestamp in UTC for the transfer creation.
        updated_at: The UNIX formatted timestamp in UTC for the transfer's latest update.
        state: The withdraw state of this transfer.
    """

    def __init__(self):
        self.id = 0
        self.type = DepositWithdraw.WITHDRAW
        self.currency = ""
        self.chain = ""
        self.tx_hash = ""
        self.amount = 0.0
        self.address = ""
        self.address_tag = ""
        self.fee = 0.0
        self.created_at = 0
        self.updated_at = 0
        self.state = WithdrawState.INVALID

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Operator Type")
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.tx_hash, format_data + "Trade Hash")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.address, format_data + "Address")
        PrintBasic.print_basic(self.address_tag, format_data + "Address Tag")
        PrintBasic.print_basic(self.fee, format_data + "Fee")
        PrintBasic.print_basic(self.state, format_data + "Withdraw State")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.updated_at, format_data + "Update Time")



from notecoin.huobi.model.algo import *
from notecoin.huobi.utils.input_checker import *


class AlgoClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def create_order(self, account_id: 'int', symbol: 'str', order_side: 'OrderSide', order_type: 'OrderType',
                     client_order_id: 'str', stop_price: 'str', order_price: 'str' = None, order_size: 'str' = None,
                     order_value: 'str' = None, time_in_force: 'str' = None, trailing_rate: 'str' = None) -> int:
        """
        Make an algo order in huobi.
        :param account_id: Account id. (mandatory)
        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param order_side: the Order side, possible values: buy,sell. (mandatory)
        :param order_type: The order type, possible values: limit, market. (mandatory)
        :param stop_price: The stop price. (mandatory)
        :param order_price: The limit price of limit order, only needed for limit order. (mandatory for buy-limit, sell-limit, buy-limit-maker and sell-limit-maker)
        :param order_size: The amount of market order only
        :param order_value: for market buy order only
        :param stop_price: Price for auto sell to get the max benefit
        :param time_in_force: gtc(invalid for orderType=market), boc(invalid orderType=market),ioc,fok(invalid for orderType=market)
        :param trailing_rate: for trailing orders only
        :param client_order_id: unique Id which is user defined and must be unique in recent 24 hours
        """

        params = self.create_order_param_check(symbol, account_id, order_side, order_type, stop_price, order_price,
                                               order_size, order_value, time_in_force, trailing_rate, client_order_id)
        from huobi.service.algo.post_create_order import PostCreateOrderService
        return PostCreateOrderService(params).request(**self.__kwargs)

    def cancel_orders(self, client_order_ids) -> CancelOrderResult:
        check_should_not_none(client_order_ids, "clientOrderIds")

        params = {
            "clientOrderIds": client_order_ids
        }
        from huobi.service.algo.post_cancel_orders import \
            PostCancelOrderService
        return PostCancelOrderService(params).request(**self.__kwargs)

    def get_open_orders(self, account_id: 'str' = None, symbol: 'str' = None, order_side: 'OrderSide' = None,
                        order_type: 'AlgoOrderType' = None, sort: 'SortDesc' = None, limit: 'int' = 100,
                        from_id: 'int' = None):

        params = {
            "accountId": account_id,
            "symbol": symbol,
            "orderSide": order_side,
            "orderType": order_type,
            "sort": sort,
            "limit": limit,
            "fromId": from_id
        }
        from huobi.service.algo.get_open_orders import GetOpenOrdersService
        return GetOpenOrdersService(params).request(**self.__kwargs)

    def get_order_history(self, symbol: 'str', order_status: 'AlgoOrderStatus', account_id: 'str' = None,
                          order_side: 'OrderSide' = None, order_type: 'AlgoOrderType' = None, start_time: 'int' = None,
                          end_time: 'int' = None, sort: 'SortDesc' = SortDesc.DESC, limit: 'int' = 100,
                          from_id: 'int' = None):

        params = {
            "symbol": symbol,
            "accountId": account_id,
            "orderSide": order_side,
            "orderType": order_type,
            "orderStatus": order_status,
            "startTime": start_time,
            "endTime": end_time,
            "sort": sort,
            "limit": limit,
            "fromId": from_id
        }
        from huobi.service.algo.get_order_history import GetOrderHistoryService
        return GetOrderHistoryService(params).request(**self.__kwargs)

    def get_order(self, client_order_id: 'str'):
        params = {
            "clientOrderId": client_order_id
        }
        from huobi.service.algo.get_order_by_cid import \
            GetOrderByClientOrderIdService
        return GetOrderByClientOrderIdService(params).request(**self.__kwargs)

    def create_order_param_check(self, symbol, account_id, order_side, order_type, stop_price, order_price,
                                 order_size, order_value, time_in_force, trailing_rate, client_order_id):
        check_symbol(symbol)
        check_should_not_none(account_id, "accountId")
        check_should_not_none(order_type, "orderType")
        check_should_not_none(order_side, "orderSide")

        if order_type == OrderType.SELL_LIMIT \
                or order_type == OrderType.BUY_LIMIT \
                or order_type == OrderType.BUY_LIMIT_MAKER \
                or order_type == OrderType.SELL_LIMIT_MAKER:
            check_should_not_none(order_price, "orderPrice")

        if time_in_force is not None:
            check_time_in_force(time_in_force)

        if order_type in [OrderType.SELL_MARKET, OrderType.BUY_MARKET]:
            order_price = None

        params = {
            "accountId": account_id,
            "symbol": symbol,
            "orderPrice": order_price,
            "orderSide": order_side,
            "orderSize": order_size,
            "orderValue": order_value,
            "timeInForce": time_in_force,
            "orderType": order_type,
            "clientOrderId": client_order_id,
            "stopPrice": stop_price,
            "trailingRate": trailing_rate
        }

        return params



from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.generic import *
from notecoin.huobi.utils import *
from notecoin.huobi.utils.json_parser import default_parse_data_as_long


class GetExchangeCurrenciesService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/common/currencys"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return data_list if len(data_list) else []

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetExchangeSymbolsService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/common/symbols"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, Symbol, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetExchangeTimestampService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/common/timestamp"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetMarketStatusService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/market-status"

        def parse(dict_data):
            return default_parse(dict_data.get("data", {}), MarketStatus)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetReferenceCurrenciesService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/reference/currencies"

        def parse(dict_data):
            ret_list = []
            data_list = dict_data.get("data", [])
            if data_list and len(data_list):
                for reference_currency in data_list:
                    reference_currency_obj = default_parse(reference_currency, ReferenceCurrency, Chain)
                    ret_list.append(reference_currency_obj)
            return ret_list

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetSystemStatusService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/api/v2/summary.json"
        kwargs["url"] = "https://status.huobigroup.com"

        def parse(dict_data):
            return dict_data

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)



class CancelOrderResult:
    """
    The result of batch cancel operation.

    :member
        accepted: The clientOrderIds accepted.
        rejected: The clientOrderIds rejected .

    """

    def __init__(self):
        self.accepted = []
        self.rejected = []

    def print_object(self, format_data=""):
        print("Success Order Counts", len(self.accepted), " accepted Order Ids : ", self.accepted)
        print("Fail Order Counts", len(self.rejected), " Rejected Order Ids : ", self.rejected)


class OrderHistoryItem:
    """
    The result of batch cancel operation.

    :member
        orderOrigTime
        lastActTime
        symbol
        source
        orderSide
        orderType
        timeInForce
        clientOrderId
        accountId
        orderPrice
        orderSize
        stopPrice
        orderStatus

    """

    def __init__(self):
        self.orderOrigTime = ""
        self.lastActTime = ""
        self.symbol = ""
        self.source = ""
        self.orderSide = ""
        self.orderType = ""
        self.timeInForce = ""
        self.clientOrderId = ""
        self.accountId = ""
        self.orderPrice = ""
        self.orderSize = ""
        self.stopPrice = ""
        self.orderStatus = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderOrigTime, format_data + "")
        PrintBasic.print_basic(self.lastActTime, format_data + "")
        PrintBasic.print_basic(self.symbol, format_data + "")
        PrintBasic.print_basic(self.source, format_data + "")
        PrintBasic.print_basic(self.orderSide, format_data + "")
        PrintBasic.print_basic(self.orderType, format_data + "")
        PrintBasic.print_basic(self.timeInForce, format_data + "")
        PrintBasic.print_basic(self.clientOrderId, format_data + "")
        PrintBasic.print_basic(self.accountId, format_data + "")
        PrintBasic.print_basic(self.orderPrice, format_data + "")
        PrintBasic.print_basic(self.orderSize, format_data + "")
        PrintBasic.print_basic(self.stopPrice, format_data + "")
        PrintBasic.print_basic(self.orderStatus, format_data + "")


class OrderListItem:
    """
    The result of batch cancel operation.

    :member
        event_type:
        symbol:
        order_id:
        trade_price:
        trade_volume:
        order_side:
        aggressor:
        trade_id:
        trade_time:
        transact_fee:
        fee_deduct:
        fee_deduct_type:
        fee_currency:
        account_id:
        source:
        order_price:
        order_size:
        client_order_id:
        order_create_time:
        order_status:
    """

    def __init__(self):
        self.eventType = ""
        self.symbol = ""
        self.orderId = ""
        self.tradePrice = ""
        self.tradeVolume = ""
        self.orderSide = ""
        self.aggressor = ""
        self.tradeId = ""
        self.tradeTime = ""
        self.transactFee = ""
        self.feeDeduct = ""
        self.feeDeductType = ""
        self.feeCurrency = ""
        self.accountId = ""
        self.source = ""
        self.orderPrice = ""
        self.orderSize = ""
        self.clientOrderId = ""
        self.orderCreateTime = ""
        self.orderStatus = ""
        self.trailingRate = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.eventType, format_data + "Event Type")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.orderId, format_data + "OrderId")
        PrintBasic.print_basic(self.tradePrice, format_data + "Trade Price")
        PrintBasic.print_basic(self.tradeVolume, format_data + "Trade Volume")
        PrintBasic.print_basic(self.orderSide, format_data + "Order Side")
        PrintBasic.print_basic(self.aggressor, format_data + "Aggressor")
        PrintBasic.print_basic(self.tradeId, format_data + "TradeId")
        PrintBasic.print_basic(self.tradeTime, format_data + "Trade Time")
        PrintBasic.print_basic(self.transactFee, format_data + "Transact Fee")
        PrintBasic.print_basic(self.feeDeduct, format_data + "Fee Deduct")
        PrintBasic.print_basic(self.feeDeductType, format_data + "Fee Deduct Type")
        PrintBasic.print_basic(self.feeCurrency, format_data + "Fee Currency")
        PrintBasic.print_basic(self.accountId, format_data + "Account Id")
        PrintBasic.print_basic(self.source, format_data + "Source")
        PrintBasic.print_basic(self.orderPrice, format_data + "Order Price")
        PrintBasic.print_basic(self.orderSize, format_data + "Order Size")
        PrintBasic.print_basic(self.clientOrderId, format_data + "Client Order Id")
        PrintBasic.print_basic(self.orderCreateTime, format_data + "Order Create Time")
        PrintBasic.print_basic(self.orderStatus, format_data + "Order Status")
        PrintBasic.print_basic(self.trailingRate, format_data + "Trailing Rate (Trailing Order Only)")



import time

from notecoin.huobi.connection import (RestApiSyncClient, SubscribeClient,
                                       WebSocketReqClient)
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.market import (Candlestick, CandlestickEvent,
                                         CandlestickReq, MarketDetail,
                                         MarketDetailEvent, MarketDetailReq,
                                         MbpFullEvent, MbpIncreaseEvent,
                                         MbpReq, PriceDepth, PriceDepthBbo,
                                         PriceDepthBboEvent, PriceDepthEvent,
                                         PriceDepthReq, Trade, TradeDetail,
                                         TradeDetailEvent, TradeDetailReq)
from notecoin.huobi.utils.channels import (kline_channel,
                                           market_detail_channel,
                                           mbp_full_channel,
                                           mbp_increase_channel,
                                           price_depth_bbo_channel,
                                           price_depth_channel,
                                           request_mbp_channel,
                                           trade_detail_channel)
from notecoin.huobi.utils.channels_request import (
    request_kline_channel, request_market_detail_channel,
    request_price_depth_channel, request_trade_detail_channel)
from notecoin.huobi.utils.json_parser import (default_parse,
                                              default_parse_list_dict)


class MarketService(RestApiSyncClient):
    def __init__(self, *args, **kwargs):

        super(MarketService, self).__init__(*args, **kwargs)

    def get_candle_stick_service(self, params):
        channel = "/market/history/kline"
        return self.request_process(HttpMethod.GET, channel, params)

    def get_history_trade_service(self, params):
        channel = "/market/history/trade"

        def parse(dict_data):
            trade_list_ret = []  # two level list, list item is list too
            data_list_outer = dict_data.get("data", [])
            if len(data_list_outer):
                for row in data_list_outer:
                    data_list_inner = row.get("data", [])
                    if len(data_list_inner):
                        for trade_info in data_list_inner:
                            trade_obj = default_parse_list_dict(trade_info, Trade, None)  # return a list
                            if trade_obj:
                                trade_list_ret.append(trade_obj)

            return trade_list_ret

        return self.request_process(HttpMethod.GET, channel, params)

    def get_market_detail_merged_service(self, params):
        channel = "/market/detail/merged"
        return self.request_process(HttpMethod.GET, channel, params)

    def get_market_detail_service(self, params):
        channel = "/market/detail"
        return self.request_process(HttpMethod.GET, channel, params)

    def get_market_tickers_service(self, params):
        channel = "/market/tickers"
        return self.request_process(HttpMethod.GET, channel, params)

    def get_market_trade_service(self, params):
        channel = "/market/trade"
        return self.request_process(HttpMethod.GET, channel, params)

    def get_price_depth_service(self, params):
        channel = "/market/depth"
        return self.request_process(HttpMethod.GET, channel, params)


class MarketServiceSocket(WebSocketReqClient):
    def __init__(self, *args, **kwargs):
        super(MarketServiceSocket, self).__init__(*args, **kwargs)

    def req_candle_stick_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        interval = params["interval"]
        from_ts_second = params.get("from_ts_second", None)
        end_ts_second = params.get("end_ts_second", None)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_kline_channel(symbol, interval, from_ts_second, end_ts_second))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, CandlestickReq, Candlestick)

        self.execute_subscribe_v1(subscription, parse, callback, error_handler)

    def req_market_detail_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_market_detail_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, MarketDetailReq, MarketDetail)

        self.execute_subscribe_v1(subscription, parse, callback, error_handler)

    def req_mbp_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        level = params["levels"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_mbp_channel(symbol, level))
                time.sleep(0.01)

        def parse(dict_data):
            return MbpReq.json_parse(dict_data)

        self.execute_subscribe_mbp(subscription,
                                   parse,
                                   callback,
                                   error_handler)

    def req_price_depth_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        step = params["step"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_price_depth_channel(symbol, step))
                time.sleep(0.01)

        def parse(dict_data):
            price_depth_event = PriceDepthReq()
            price_depth_event.id = dict_data.get("id")
            price_depth_event.rep = dict_data.get("rep")
            data = dict_data.get("data", {})
            price_depth_obj = PriceDepth.json_parse(data)
            price_depth_event.data = price_depth_obj
            return price_depth_event

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def req_trade_detail_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_trade_detail_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, TradeDetailReq, TradeDetail)

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)


class MarketServiceSub(SubscribeClient):
    def __init__(self, *args, **kwargs):
        super(MarketServiceSub, self).__init__(*args, **kwargs)

    def sub_candle_stick_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        interval = params["interval"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(kline_channel(symbol, interval))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, CandlestickEvent, Candlestick)

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def sub_market_detail_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(market_detail_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, MarketDetailEvent, MarketDetail)

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def sub_mbp_full_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        level = params["levels"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(mbp_full_channel(symbol, level))
                time.sleep(0.01)

        def parse(dict_data):
            return MbpFullEvent.json_parse(dict_data)

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def sub_mbp_increase_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        level = params["levels"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(mbp_increase_channel(symbol, level))
                time.sleep(0.01)

        def parse(dict_data):
            return MbpIncreaseEvent.json_parse(dict_data)

        self.execute_subscribe_mbp(subscription,
                                   parse,
                                   callback,
                                   error_handler)

    def sub_price_depth_bbo_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(price_depth_bbo_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            return default_parse(dict_data, PriceDepthBboEvent, PriceDepthBbo)

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def sub_price_depth_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]
        step = params["step"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(price_depth_channel(symbol, step))
                time.sleep(0.01)

        def parse(dict_data):
            price_depth_event_obj = PriceDepthEvent()
            price_depth_event_obj.ch = dict_data.get("ch", "")
            tick = dict_data.get("tick", "")
            price_depth_event_obj.tick = PriceDepth.json_parse(tick)
            return price_depth_event_obj

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)

    def sub_trade_detail_service(self, callback, error_handler, params):
        symbol_list = params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(trade_detail_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            tick = dict_data.get("tick", {})
            trade_detail_event = default_parse(tick, TradeDetailEvent, TradeDetail)
            trade_detail_event.ch = dict_data.get("ch", "")
            return trade_detail_event

        self.execute_subscribe_v1(subscription,
                                  parse,
                                  callback,
                                  error_handler)




from notecoin.huobi.constant import *
from notecoin.huobi.utils.json_parser import default_parse_list_dict, fill_obj


class BatchCancelCount:
    """
    The result of batch cancel operation.

    :member
        success_count: The number of cancel request sent successfully.
        failed_count: The number of cancel request failed.
        next_id:next open order id
    """

    def __init__(self):
        self.success_count = 0
        self.failed_count = 0
        self.next_id = -1

    def print_object(self, format_data=""):
        from huobi.utils import PrintBasic
        PrintBasic.print_basic(self.success_count, format_data + "Success Count")
        PrintBasic.print_basic(self.failed_count, format_data + "Failed Count")
        PrintBasic.print_basic(self.next_id, format_data + "Next Open Order ID")


class BatchCancelResult:
    """
    The result of batch cancel operation.

    :member
        success_count: The number of cancel request sent successfully.
        failed_count: The number of cancel request failed.

    """

    def __init__(self):
        self.success = []
        self.failed = []

    def print_object(self, format_data=""):
        print("Success Order Counts", len(self.success), " Success Order Ids : ", self.success)
        print("Fail Order Counts", len(self.failed), " Fail Order Ids : ", self.failed)


class BatchCreateOrder:
    """
    batch create order result

    :member
        order_id: The transfer id.
        client_order_id: The crypto currency to deposit.
        err_code: The on-chain transaction hash.
        err_msg: The number of crypto asset transferred in its minimum unit.

    """

    def __init__(self):
        self.order_id = 0
        self.client_order_id = ""
        self.err_code = ""
        self.err_msg = ""

    def print_object(self, format_data=""):
        from huobi.utils import PrintBasic
        PrintBasic.print_basic(self.order_id, format_data + "Order Id")
        PrintBasic.print_basic(self.client_order_id, format_data + "Client Order Id")
        PrintBasic.print_basic(self.err_code, format_data + "Error Code")
        PrintBasic.print_basic(self.err_msg, format_data + "Error Message")


class FeeRate:
    """
    The account information for spot account, margin account etc.

    :member
        symbol: The symbol, like "btcusdt".
        maker_fee: maker fee rate
        taker_fee: taker fee rate

    """

    def __init__(self):
        self.symbol = ""
        self.maker_fee = ""
        self.taker_fee = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.maker_fee, format_data + "Maker Fee")
        PrintBasic.print_basic(self.taker_fee, format_data + "Taker Fee")


class MatchResult:
    """
    The match result information.

    :member
        created_timestamp: The UNIX formatted timestamp in UTC when the match and fill is done.
        filled_amount: The amount which has been filled.
        filled_fees: The transaction fee paid so far.
        id: The internal id.
        match_id: The match id of this match.
        order_id: The order id of this order.
        price: The limit price of limit order.
        source: The source where the order was triggered, possible values: sys, web, api, app.
        symbol: The symbol, like "btcusdt".
        type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit,
            buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok, buy-stop-limit-fok, sell-stop-limit-fok.
        filled_points: deduct points
        fee_deduct_currency: deduct type, it means deduct from HT/ HT points / or other currency
        fee_currency:
    """

    def __init__(self):
        self.created_at = 0
        self.filled_amount = 0.0
        self.filled_fees = 0.0
        self.id = 0
        self.match_id = 0
        self.order_id = 0
        self.price = 0.0
        self.source = OrderSource.INVALID
        self.symbol = ""
        self.type = OrderType.INVALID
        self.role = ""
        self.filled_points = ""
        self.fee_deduct_currency = ""
        self.fee_currency = ""
        self.fee_deduct_state = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.filled_amount, format_data + "Fill Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Fill Fee")
        PrintBasic.print_basic(self.filled_points, format_data + "Fill Points")
        PrintBasic.print_basic(self.match_id, format_data + "Match ID")
        PrintBasic.print_basic(self.order_id, format_data + "Order ID")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.source, format_data + "Source")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.role, format_data + "Role")
        PrintBasic.print_basic(self.fee_deduct_currency, format_data + "Fee Deduct Currency")
        PrintBasic.print_basic(self.fee_currency, format_data + "Fee Currency")
        PrintBasic.print_basic(self.fee_deduct_state, format_data + "Fee Deduct State")


class OrderDetailReq:
    """
    The order update received by subscription of order update.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic : request topic
        client_req_id : client request id
        data: The order detail.

    """

    def __init__(self):
        self.ts = 0
        self.topic = ""
        self.cid = ""
        self.data = OrderListItem()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.cid, format_data + "Client Req ID")
        PrintBasic.print_basic(self.topic, format_data + "Topic")
        self.data.print_object("\t")


class OrderListItem:
    """
    The order update received by request of order list.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic: request topic
        client_req_id: client request ID
        order_list : order list

    """

    def __init__(self):
        self.id = 0
        self.symbol = ""
        self.account_id = 0
        self.amount = 0.0
        self.price = 0.0
        self.created_at = 0
        self.type = OrderType.INVALID
        self.finished_at = 0
        self.source = OrderSource.INVALID
        self.state = OrderState.INVALID
        self.canceled_at = 0
        self.filled_amount = 0.0
        self.filled_cash_amount = 0.0
        self.filled_fees = 0.0
        self.stop_price = 0.0
        self.operator = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.finished_at, format_data + "Finish Time")
        PrintBasic.print_basic(self.source, format_data + "Order Source")
        PrintBasic.print_basic(self.state, format_data + "Order State")
        PrintBasic.print_basic(self.canceled_at, format_data + "Cancel Time")
        PrintBasic.print_basic(self.filled_amount, format_data + "Filled Amount")
        PrintBasic.print_basic(self.filled_cash_amount, format_data + "Filled Cash Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Filled Fees")
        PrintBasic.print_basic(self.stop_price, format_data + "Stop Price")
        PrintBasic.print_basic(self.operator, format_data + "Operator")


class OrderListReq:
    """
    The order update received by request of order list.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        topic: request topic
        data : OrderListItem

    """

    def __init__(self):
        self.ts = 0
        self.topic = ""
        self.data = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.topic, format_data + "Channel")
        print("Order List as below : count " + str(len(self.data)))
        if len(self.data):
            for orderlistitem_obj in self.data:
                orderlistitem_obj.print_object("\t ")
                print()


class OrderUpdateEvent:
    """
    The order update received by subscription of order update.

    :member
        ch: The symbol you subscribed.
        data: The order detail.

    """

    def __init__(self):
        self.ch = ""
        self.data = OrderUpdate()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Topic")

        orderupdate = self.data
        orderupdate.print_object()


class OrderUpdate:
    """
    The detail order information.

    :member
        orderId: The order id.
        tradePrice: trade price
        tradeVolume: trade volume
        tradeId: Id record for trade
        tradeTime: trade timestamp (ms)
        aggressor: true (taker), false (maker)
        remainAmt: Remaining amount (for buy-market order it's remaining value)
        orderStatus: Order status, valid value: partial-filled, filled
        clientOrderId: Client order ID (if any)
        eventType: Event type, valid value: trade
        symbol: The symbol, like "btcusdt".
        type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit, buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok.
    """

    def __init__(self):
        self.orderId = 0
        self.tradePrice = ""
        self.tradeVolume = ""
        self.tradeId = 0
        self.tradeTime = 0
        self.aggressor = False
        self.remainAmt = ""
        self.orderStatus = OrderState.INVALID
        self.clientOrderId = ""
        self.eventType = ""
        self.symbol = ""
        self.type = OrderType.INVALID
        self.accountId = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderId, format_data + "Order Id")
        PrintBasic.print_basic(self.tradePrice, format_data + "Trade Price")
        PrintBasic.print_basic(self.tradeVolume, format_data + "Trade Volume")
        PrintBasic.print_basic(self.tradeId, format_data + "Trade Id")
        PrintBasic.print_basic(self.tradeTime, format_data + "Trade Timestamp")
        PrintBasic.print_basic(self.aggressor, format_data + "is Taker")
        PrintBasic.print_basic(self.orderStatus, format_data + "Order State")
        PrintBasic.print_basic(self.clientOrderId, format_data + "Client Order Id")
        PrintBasic.print_basic(self.eventType, format_data + "Event Type")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.accountId, format_data + "Account Id")


class Order:
    """
    The detail order information.

    :member
        amount: The amount of base currency in this order.
        price: The limit price of limit order.
        created_timestamp: The UNIX formatted timestamp in UTC when the order was created.
        canceled_timestamp: The UNIX formatted timestamp in UTC when the order was canceled, if not canceled then has value of 0.
        finished_timestamp: The UNIX formatted timestamp in UTC when the order was changed to a final state. This is not the time the order is matched.
        order_id: The order id.
        symbol: The symbol, like "btcusdt".
        order_type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit, buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok, buy-stop-limit-fok, sell-stop-limit-fok.
        filled_amount: The amount which has been filled.
        filled_cash_amount: The filled total in quote currency.
        filled_fees: The transaction fee paid so far.
        source: The source where the order was triggered, possible values: sys, web, api, app.
        state: The order state: submitted, partial-filled, cancelling, filled, canceled.
        stop_price : stop price used for buy-stop-limit，sell-stop-limit
        operator : only [gte] and [lte] to trigger buy-stop-limit，sell-stop-limit
    """

    def __init__(self):
        self.id = 0
        self.symbol = ""
        self.account_id = 0
        self.amount = 0.0
        self.price = 0.0
        self.created_at = 0
        self.canceled_at = 0
        self.finished_at = 0
        self.type = OrderType.INVALID
        self.filled_amount = 0.0
        self.filled_cash_amount = 0.0
        self.filled_fees = 0.0
        self.source = OrderSource.INVALID
        self.state = OrderState.INVALID
        self.client_order_id = ""
        self.stop_price = ""
        self.next_time = 0
        self.operator = ""

    @staticmethod
    def json_parse(json_data):
        order = fill_obj(json_data, Order)
        order.filled_amount = json_data.get("filled-amount", json_data.get("field-amount", 0))
        order.filled_cash_amount = json_data.get("filled-cash-amount", json_data.get("field-cash-amount", 0))
        order.filled_fees = json_data.get("filled-fees", json_data.get("field-fees", 0))
        return order

    @staticmethod
    def json_parse_list(json_data):
        if json_data and len(json_data):
            order_list = list()
            for idx, row in enumerate(json_data):
                order_item = Order.json_parse(row)
                order_list.append(order_item)
            return order_list

        return list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Order Id")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.canceled_at, format_data + "Cancel Time")
        PrintBasic.print_basic(self.finished_at, format_data + "Finish Time")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.filled_amount, format_data + "Filled Amount")
        PrintBasic.print_basic(self.filled_cash_amount, format_data + "Filled Cash Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Filled Fees")
        #PrintBasic.print_basic(self.account_type, format_data + "Account Type")
        PrintBasic.print_basic(self.source, format_data + "Order Source")
        PrintBasic.print_basic(self.state, format_data + "Order State")
        PrintBasic.print_basic(self.client_order_id, format_data + "Client Order Id")
        PrintBasic.print_basic(self.stop_price, format_data + "Stop Price")
        PrintBasic.print_basic(self.operator, format_data + "Operator")
        PrintBasic.print_basic(self.next_time, format_data + "Next Time")


class TradeClearingEvent:
    """
    subscribe trading clearing information

    :member
        action: current is "sub" for subscribe
        ch: subscribe topic.
        data: data detail in TradeClearing.
    """

    def __init__(self):
        self.action = ""
        self.ch = ""
        self.seq = 0
        self.data = TradeClearing()

    @staticmethod
    def json_parse(data_json):
        event_obj = TradeClearingEvent()
        event_obj.action = data_json.get("action")
        event_obj.ch = data_json.get("ch")
        event_obj.seq = data_json.get("seq", 0)
        event_obj.data = TradeClearing.json_parse(data_json.get("data", {}))
        return event_obj

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        self.data.print_object()


class TradeClearing:
    """
    The detail order information.

    :member
        order_id: The order id.
        symbol: The symbol, like "btcusdt".
        tradePrice: trade price.
        tradeVolume: trade Volume.
        orderSide: order Side, more to see OrderSide
        orderType: order type, more to see OrderType
        aggressor: is aggressor, only true or false
        tradeId: trade ID.
        tradeTime: trade Time.
        transactFee: transact Fee.
        feeDeduct: Deduct Fee.
        feeDeductType: fee Deduct Type, current only support ht and point
    """

    def __init__(self):
        self.symbol = ""
        self.orderId = 0
        self.tradePrice = ""
        self.tradeVolume = ""
        self.orderSide = OrderSide.INVALID
        self.orderType = OrderType.INVALID
        self.aggressor = False
        self.tradeId = 0
        self.tradeTime = 0
        self.transactFee = ""
        self.feeDeduct = ""
        self.feeDeductType = FeeDeductType.INVALID

    @staticmethod
    def json_parse(json_data):
        if json_data.get("orderId", None):
            return default_parse_list_dict(json_data, TradeClearing)
        else:
            return TradeClearing()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderId, format_data + "Order Id")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.tradePrice, format_data + "Trade Price")
        PrintBasic.print_basic(self.tradeVolume, format_data + "Trade Volume")
        PrintBasic.print_basic(self.orderSide, format_data + "Order Side")
        PrintBasic.print_basic(self.orderType, format_data + "Order Type")
        PrintBasic.print_basic(self.aggressor, format_data + "is Taker")
        PrintBasic.print_basic(self.tradeId, format_data + "Trade Id")
        PrintBasic.print_basic(self.tradeTime, format_data + "Trade Time")
        PrintBasic.print_basic(self.transactFee, format_data + "Transact Fee")
        PrintBasic.print_basic(self.feeDeduct, format_data + "Fee Deduct")
        PrintBasic.print_basic(self.feeDeductType, format_data + "Fee Deduct Type")


class TransactFeeRate:
    """
    The transact fee rate.

    :member
        symbol: symbol like "btcusdt"
        makerFeeRate: maker fee rate
        takerFeeRate: taker fee rate
        actualMakerRate: actual maker fee rate
        actualTakerRate: actual taker fee rate
    """

    def __init__(self):
        self.symbol = ""
        self.makerFeeRate = ""
        self.takerFeeRate = ""
        self.actualMakerRate = ""
        self.actualTakerRate = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.makerFeeRate, format_data + "makerFeeRate")
        PrintBasic.print_basic(self.takerFeeRate, format_data + "takerFeeRate")
        PrintBasic.print_basic(self.actualMakerRate, format_data + "actualMakerRate")
        PrintBasic.print_basic(self.actualTakerRate, format_data + "actualTakerRate")



from typing import List

from notecoin.huobi.constant import *
from notecoin.huobi.utils.print_mix_object import PrintBasic


class Trade:
    """
    The trade information with price and amount etc.

    :member
        price: The trading price in quote currency.
        amount: The trading volume in base currency.
        trade_id: The unique trade id of this trade.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        direction: The direction of the taker trade: 'buy' or 'sell'.
    """

    def __init__(self):
        self.price = 0.0
        self.amount = 0.0
        self.trade_id = 0
        self.ts = 0
        self.direction = ""

    def print_object(self, format_data=""):
        print(self.__dict__)


class TradeDetail:
    """
    The trade information with price and amount etc.

    :member
        price: The trading price in quote currency.
        amount: The trading volume in base currency.
        tradeId: The unique trade id of this trade.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        direction: The direction of the taker trade: 'buy' or 'sell'.
    """

    def __init__(self):
        self.price = 0.0
        self.amount = 0.0
        self.tradeId = 0
        self.ts = 0
        self.direction = ""

    def print_object(self, format_data=""):
        print(self.__dict__)


class TradeDetailEvent:
    """
    The trade received by subscription of trade.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        trade_list: The trade list. The content is Trade class.
    """

    def __init__(self):
        self.ch = ""
        self.id = 0
        self.ts = 0
        self.data: List[Trade] = []

    def print_object(self, format_data=""):
        print(self.__dict__)
        if len(self.data):
            for trade_detail in self.data:
                trade_detail.print_object()


class TradeDetailReq:
    """
    The trade received by subscription of trade.

    :member
        rep: The Channel you subscribed.
        trade_list: The trade list. The content is Trade class.
    """

    def __init__(self):
        self.rep = ""
        self.data = list()

    def print_object(self, format_data=""):
        print(self.__dict__)
        if len(self.data):
            for trade_detail in self.data:
                trade_detail.print_object()


class CandlestickEvent:
    """
    The candlestick/kline data received by subscription of candlestick/kline.

    :member
        ch: the topic you subscribed
        ts: the UNIX formatted timestamp generated by server in UTC.
        tick: the data of candlestick/kline.
    """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.tick = Candlestick()

    def print_object(self, format_data=""):
        print(self.__dict__)
        if self.tick:
            self.tick.print_object()


class CandlestickReq:
    """
    The candlestick/kline data received by subscription of candlestick/kline.

    :member
        rep: the Channel or topic you subscribed.
        id: the UNIX formatted timestamp generated by server in UTC.
        data: the data of candlestick/kline.

    """

    def __init__(self):
        self.rep = ""
        self.id = 0
        self.data = list()

    def print_object(self, format_data=""):
        print(self.__dict__)
        if len(self.data):
            for row in self.data:
                row.print_object()


class Candlestick:
    """
    The candlestick/kline data.

    :member
        id : keep the original timestamp
        timestamp: The UNIX formatted timestamp in UTC.
        high: The high price.
        low: The low price.
        open: The opening price.
        close: The closing price.
        amount: The aggregated trading volume in USDT.
        count: The number of completed trades. it returns 0 when get ETF candlestick
        vol: The trading volume in base currency.

    """

    def __init__(self):
        self.id = 0
        #self.timestamp = 0
        self.high = 0.0
        self.low = 0.0
        self.open = 0.0
        self.close = 0.0
        self.amount = 0.0
        self.count = 0
        self.vol = 0.0  # self.volume = 0.0

    def print_object(self, format_data=""):
        print(self.__dict__)


class DepthEntry:
    """
    An depth entry consisting of price and amount.

    :member
        price: The price of the depth.
        amount: The amount of the depth.
    """

    def __init__(self):
        self.price = 0.0
        self.amount = 0.0

    @staticmethod
    def json_parse(data_array):
        entry = DepthEntry()
        entry.price = data_array[0]
        entry.amount = data_array[1]
        return entry

    def print_object(self, format_data=""):
        print(self.__dict__)


class LastTradeAndBestQuote:
    """
    The last trade and best bid/ask.

    :member
        last_trade_price: The last trade price.
        last_trade_amount: The last trade amount.
        ask_price: The best ask price.
        ask_amount: The best ask amount.
        bid_price: The best bid price.
        bid_amount: The best bid amount.

    """

    def __init__(self):
        self.last_trade_price = 0.0
        self.last_trade_amount = 0.0
        self.ask_price = 0.0
        self.ask_amount = 0.0
        self.bid_price = 0.0
        self.bid_amount = 0.0


class MarketDetailEvent:
    """
     The 24H trade statistics received by subscription of trade statistics.

     :member
         Channel: topic you subscribe, include symbol.
         timestamp: The UNIX formatted timestamp generated by server in UTC.
         trade_statistics: The trade statistics.
     """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.tick = MarketDetail()

    def print_object(self, format_data=""):
        print(self.__dict__)


class MarketDetailMerged:
    """
    The best bid/ask consisting of price and amount.

    :member
        timestamp: The Unix formatted timestamp in UTC.
        bid_price: The best bid price.
        bid_amount: The best bid amount.
        ask_price: The best ask price.
        ask_amount: The best ask amount.

    """

    def __init__(self):
        self.amount = 0
        self.open = 0.0
        self.close = 0.0
        self.high = 0.0
        self.id = 0
        self.count = 0.0
        self.low = 0.0
        self.version = 0
        self.ask = []
        self.vol = 0.0
        self.bid = []

    def print_object(self, format_data=""):
        print(self.__dict__)


class MarketDetailReq:
    """
     The 24H trade statistics received by request of trade statistics only once.

     :member
         rep: The topic you subscribed.
         ts: The UNIX formatted timestamp generated by server in UTC.
         trade_statistics: The trade statistics.
     """

    def __init__(self):
        self.rep = 0
        self.ts = 0
        self.data = MarketDetail()

    def print_object(self, format_data=""):
        print(self.__dict__)


class MarketDetail:
    """
    The summary of trading in the market for the last 24 hours

    :member
        id: response ID
        open: The opening price of last 24 hours.
        close: The last price of last 24 hours.
        amount: The aggregated trading volume in USDT.
        high: The high price of last 24 hours.
        low: The low price of last 24 hours.
        count: The number of completed trades.
        volume: The trading volume in base currency of last 24 hours.
        version: inner data
    """

    def __init__(self):
        self.id = 0
        self.open = 0.0
        self.close = 0.0
        self.amount = 0.0
        self.high = 0.0
        self.low = 0.0
        self.count = 0
        self.vol = 0.0
        self.version = 0

    def print_object(self, format_data=""):
        print(self.__dict__)


class MarketTicker:
    """
    The ticker information.

    :member
        amount: The aggregated trading volume in last 24 hours (rotating 24h).
        count: The number of completed trades of last 24 hours (rotating 24h).
        open: The opening price of a nature day (Singapore time).
        close: The last price of a nature day (Singapore time).
        low: The low price of a nature day (Singapore time).
        high: The high price of a nature day (Singapore time).
        vol: The aggregated trading value in last 24 hours (rotating 24h).
        symbol: The trading symbol of this object, e.g. btcusdt, bccbtc.
        bid: Best bid price.
        bidSize: Best bid size.
        ask: Best ask price.
        askSize: Best ask size.
    """

    def __init__(self):
        self.amount = 0.0
        self.count = 0
        self.open = 0.0
        self.close = 0.0
        self.low = 0.0
        self.high = 0.0
        self.vol = 0.0
        self.symbol = ""
        self.bid = 0.0
        self.bidSize = 0.0
        self.ask = 0.0
        self.askSize = 0.0

    def print_object(self, format_data=""):
        print(self.__dict__)


class MbpFullEvent:
    """
    full price depth.

    :member
        ch: Topic of subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.

    """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.data = Mbp()

    @staticmethod
    def json_parse(json_data):
        mbp_event = MbpFullEvent()
        mbp_event.ts = json_data.get("ts")
        mbp_event.ch = json_data.get("ch")
        mbp = Mbp.json_parse(json_data.get("tick", {}))
        mbp_event.data = mbp
        return mbp_event

    def print_object(self, format_data=""):
        print(self.__dict__)
        self.data.print_object()


class MbpIncreaseEvent:
    """
    increasement of price depth.

    :member
        ch: Topic of subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.

    """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.data = Mbp()

    @staticmethod
    def json_parse(json_data):
        mbp_event = MbpIncreaseEvent()
        mbp_event.ts = json_data.get("ts")
        mbp_event.ch = json_data.get("ch")
        mbp = Mbp.json_parse(json_data.get("tick", {}))
        mbp_event.data = mbp
        return mbp_event

    def print_object(self, format_data=""):
        print(self.__dict__)
        self.data.print_object()


class MbpReq:
    """
    The market price depth.

    :member
        rep: request Topic
        id: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.
    """

    def __init__(self):
        self.rep = ""
        self.id = ""

        self.data = Mbp()

    @staticmethod
    def json_parse(data_json):
        mbp_event = MbpReq()
        mbp_event.id = data_json.get("id")
        mbp_event.rep = data_json.get("rep")
        data = data_json.get("data", {})
        mbp = Mbp.json_parse(data)
        mbp_event.data = mbp
        return mbp_event

    def print_object(self, format_data=""):
        print(self.__dict__)

        self.data.print_object()


class Mbp:
    """
    Increasement of price depth information.

    :member
        seqNum: current seqNum.
        prevSeqNum: previous seqNum.
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """

    def __init__(self):
        self.seqNum = 0
        self.prevSeqNum = 0
        self.bids = list()
        self.asks = list()

    @staticmethod
    def json_parse(json_data):
        mbp = Mbp()
        bid_list = list()
        mbp.seqNum = json_data.get("seqNum", 0)
        # prevSeqNum only for increased subscribe, request doesn't have this value
        mbp.prevSeqNum = json_data.get("prevSeqNum", 0)
        for item in json_data.get("bids", []):
            depth_entry = DepthEntry()
            depth_entry.price = item[0]
            depth_entry.amount = item[1]
            bid_list.append(depth_entry)
        ask_list = list()
        for item in json_data.get("asks", []):
            depth_entry = DepthEntry()
            depth_entry.price = item[0]
            depth_entry.amount = item[1]
            ask_list.append(depth_entry)
        mbp.bids = bid_list
        mbp.asks = ask_list

        return mbp

    def print_object(self, format_data=""):
        print(self.__dict__)
        for entry in self.bids:
            PrintBasic.print_basic(str(entry.price) + ", amount: " + str(entry.amount), format_data + "Bids price: ")

        for entry in self.asks:
            PrintBasic.print_basic(str(entry.price) + ", amount: " + str(entry.amount), format_data + "Asks price: ")


class PriceDepthBboEvent:
    """
    The price depth received by subscription of price depth.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.

    """

    def __init__(self):
        self.ts = 0
        self.ch = ""
        self.tick = PriceDepthBbo()

    def print_object(self, format_data=""):
        print(self.__dict__)
        self.tick.print_object(format_data)


class PriceDepthBbo:
    """
    The price depth information.

    :member
        timestamp: The UNIX formatted timestamp in UTC.
        bid: the first bid near trade value.
        bidSize: the bid size.
        ask: The first ask near trade value.
        askSize: the ask size.
        quoteTime : quote time
        symbol : trade symbol


    """

    def __init__(self):
        self.seqId = 0
        self.ask = 0.0
        self.askSize = 0.0
        self.bid = 0.0
        self.bidSize = 0.0
        self.quoteTime = 0
        self.symbol = ""

    def print_object(self, format_data=""):
        print(self.__dict__)


class PriceDepthEvent:
    """
    The price depth information.

    :member
        ts: The UNIX formatted timestamp in UTC.
        version:
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """

    def __init__(self):
        self.ch = ""
        self.tick = PriceDepth()

    def print_object(self, format_data=""):
        print(self.__dict__)
        self.tick.print_object("\t")


class PriceDepthReq:
    """
    The price depth information.

    :member
        ts: The UNIX formatted timestamp in UTC.
        version:
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """

    def __init__(self):
        self.rep = ""
        self.data = PriceDepth()

    def print_object(self, format_data=""):
        print(self.__dict__)
        self.data.print_object("\t")


class PriceDepth:
    """
    The price depth information.

    :member
        ts: The UNIX formatted timestamp in UTC.
        version:
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """

    def __init__(self):
        self.ts = 0
        self.version = 0
        self.bids = list()
        self.asks = list()

    @staticmethod
    def json_parse(dict_data):
        price_depth_obj = PriceDepth()
        price_depth_obj.ts = dict_data.get("ts")
        price_depth_obj.version = dict_data.get("version")
        bid_list = list()
        bids_array = dict_data.get("bids", [])
        for item in bids_array:
            depth_entry = DepthEntry.json_parse(item)
            bid_list.append(depth_entry)
        ask_list = list()
        asks_array = dict_data.get("asks", [])
        for item in asks_array:
            depth_entry = DepthEntry.json_parse(item)
            ask_list.append(depth_entry)
        price_depth_obj.bids = bid_list
        price_depth_obj.asks = ask_list

        return price_depth_obj

    def print_object(self, format_data=""):
        print(self.__dict__)
        if len(self.bids):
            i = 0
            print(format_data, "---- Top " + str(len(self.bids)) + " bids ----")
            for entry in self.bids:
                i = i + 1
                print(format_data, str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))
                # print()

        if len(self.asks):
            i = 0
            print(format_data, "---- Top " + str(len(self.asks)) + " asks ----")
            for entry in self.asks:
                i = i + 1
                print(format_data, str(i) + ": price: " + str(entry.price) + ", amount: " + str(entry.amount))
                # print()
                #



from notecoin.huobi.constant import *


class Chain:
    """
    The Huobi Chain.

    :member
        chain: Chain name
        numOfConfirmations: Number of confirmations required for deposit success (trading & withdrawal allowed once reached)
        numOfFastConfirmations: Number of confirmations required for quick success (trading allowed but withdrawal disallowed once reached)
        minDepositAmt: Minimal deposit amount in each request
        depositStatus: Deposit status	allowed,prohibited
        minWithdrawAmt: Minimal withdraw amount in each request.
        maxWithdrawAmt : Maximum withdraw amount in each request
        withdrawQuotaPerDay : Maximum withdraw amount in a day
        withdrawQuotaPerYear : Maximum withdraw amount in a year
        withdrawQuotaTotal : Maximum withdraw amount in total
        withdrawPrecision : Withdraw amount precision
        withdrawFeeType : Type of withdraw fee (only one type can be applied to each currency)

        transactFeeWithdraw : Withdraw fee in each request (only applicable to withdrawFeeType = fixed)
        minTransactFeeWithdraw : Minimal withdraw fee in each request (only applicable to withdrawFeeType = circulated)
        maxTransactFeeWithdraw : Maximum withdraw fee in each request (only applicable to withdrawFeeType = circulated or ratio)
        transactFeeRateWithdraw : Withdraw fee in each request (only applicable to withdrawFeeType = ratio)
        withdrawStatus : Withdraw status
    """

    def __init__(self):
        self.chain = ""
        self.baseChain = ""
        self.baseChainProtocol = ""
        self.numOfConfirmations = 0
        self.numOfFastConfirmations = 0
        self.depositStatus = ChainDepositStatus.INVALID
        self.minDepositAmt = 0
        self.withdrawStatus = ChainWithdrawStatus.INVALID
        self.minWithdrawAmt = 0
        self.withdrawPrecision = 0
        self.maxWithdrawAmt = 0.0
        self.withdrawQuotaPerDay = 0.0
        self.withdrawQuotaPerYear = 0.0
        self.withdrawQuotaTotal = 0.0
        self.withdrawFeeType = ""
        self.transactFeeWithdraw = 0.0
        self.minTransactFeeWithdraw = 0.0
        self.maxTransactFeeWithdraw = 0.0
        self.transactFeeRateWithdraw = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.baseChain, format_data + "Base Chain")
        PrintBasic.print_basic(self.baseChainProtocol, format_data + "Base Chain Protocol")
        PrintBasic.print_basic(self.numOfConfirmations, format_data + "numOfConfirmations")
        PrintBasic.print_basic(self.numOfFastConfirmations, format_data + "numOfFastConfirmations")
        PrintBasic.print_basic(self.depositStatus, format_data + "depositStatus")
        PrintBasic.print_basic(self.minDepositAmt, format_data + "minDepositAmount")
        PrintBasic.print_basic(self.withdrawStatus, format_data + "withdrawStatus")
        PrintBasic.print_basic(self.minWithdrawAmt, format_data + "minWithdrawAmount")
        PrintBasic.print_basic(self.withdrawPrecision, format_data + "withdrawPrecision")
        PrintBasic.print_basic(self.maxWithdrawAmt, format_data + "maxWithdrawAmount")
        PrintBasic.print_basic(self.withdrawQuotaPerDay, format_data + "withdrawQuotaPerDay")
        PrintBasic.print_basic(self.withdrawQuotaPerYear, format_data + "withdrawQuotaPerYear")
        PrintBasic.print_basic(self.withdrawQuotaTotal, format_data + "withdrawQuotaTotal")
        PrintBasic.print_basic(self.withdrawFeeType, format_data + "withdrawFeeType")
        PrintBasic.print_basic(self.transactFeeWithdraw, format_data + "transactFeeWithdraw")
        PrintBasic.print_basic(self.minTransactFeeWithdraw, format_data + "minTransactFeeWithdraw")
        PrintBasic.print_basic(self.maxTransactFeeWithdraw, format_data + "maxTransactFeeWithdraw")
        PrintBasic.print_basic(self.transactFeeRateWithdraw, format_data + "transactFeeRateWithdraw")


class ExchangeInfo:
    """
    The Huobi supported the symbols and currencies.

    :member
        symbol_list: The symbol list. The content is Symbol class.
        currencies: The currency list. The content is string value.
    """

    def __init__(self):
        self.symbol_list = list()
        self.currencies = list()


class MarketStatus:
    """
    The Huobi market status info.

    :member
        marketStatus: .
        haltStartTime: .
        haltEndTime: .
        haltReason:
        affectedSymbols:
    """

    def __init__(self):
        self.marketStatus = MarketStatus.NORMAL
        self.haltStartTime = -1
        self.haltEndTime = -1
        self.affectedSymbols = ""


class ReferenceCurrency:
    """
    The Huobi supported static reference information for each currency.

    :member
        currency: currency
        instStatus: Instrument status
        chains: chain list
    """

    def __init__(self):
        self.currency = ""
        self.instStatus = InstrumentStatus.INVALID
        self.chains = []

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.instStatus, format_data + "Instrument Status")
        if self.chains and len(self.chains):
            for chain_obj in self.chains:
                chain_obj.print_object("\t")
                print()


class Symbol:
    """
    The Huobi supported symbols.

    :member
        base_currency: The base currency in a trading symbol.
        quote_currency: The quote currency in a trading symbol.
        price_precision: The quote currency precision when quote price (decimal places).
        amount_precision: The base currency precision when quote amount (decimal places).
        symbol_partition: The trading section, possible values: [main，innovation，bifurcation].
        symbol: The symbol, like "btcusdt".
        state : trade status, maybe one in [online，offline,suspend]
        value_precision : value precision
        min_order_amt : minimum volume limit only used in limit-order and sell-market order
        max_order_amt : Maximum volume
        min_order_value : Minimum order amount
        leverage_ratio : Leverage ratio for symbol
        limit_order_min_order_amt: Minimum order amount of limit order in base currency (NEW)
        limit_order_max_order_amt: Max order amount of limit order in base currency (NEW)
        sell_market_min_order_amt: Minimum order amount of sell-market order in base currency (NEW)
        sell_market_max_order_amt: Max order amount of sell-market order in base currency (NEW)
        buy_market_max_order_amt: Max order value of buy-market order in quote currency (NEW)
        max_order_value: Max order value of limit order and buy-market order in usdt (NEW)

    """

    def __init__(self):
        self.base_currency = ""
        self.quote_currency = ""
        self.price_precision = 0
        self.amount_precision = 0
        self.symbol_partition = ""
        self.symbol = ""
        self.state = ""
        self.value_precision = 0
        self.min_order_amt = ""
        self.max_order_amt = ""
        self.min_order_value = ""
        self.leverage_ratio = 0
        self.limit_order_min_order_amt = 0
        self.limit_order_max_order_amt = 0
        self.sell_market_min_order_amt = 0
        self.sell_market_max_order_amt = 0
        self.buy_market_max_order_value = 0
        self.max_order_value = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.base_currency, format_data + "Base Currency")
        PrintBasic.print_basic(self.quote_currency, format_data + "Quote Currency")
        PrintBasic.print_basic(self.price_precision, format_data + "Price Precision")
        PrintBasic.print_basic(self.amount_precision, format_data + "Amount Precision")
        PrintBasic.print_basic(self.symbol_partition, format_data + "Symbol Partition")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.state, format_data + "State")
        PrintBasic.print_basic(self.value_precision, format_data + "Value Precision")
        PrintBasic.print_basic(self.min_order_amt, format_data + "Min Order Amount")
        PrintBasic.print_basic(self.max_order_amt, format_data + "Max Order Amount")
        PrintBasic.print_basic(self.min_order_value, format_data + "Min Order Value")
        PrintBasic.print_basic(self.leverage_ratio, format_data + "Leverage Ratio")
        PrintBasic.print_basic(self.limit_order_min_order_amt, format_data + "Minimum order amount (Limit Order)")
        PrintBasic.print_basic(self.limit_order_max_order_amt, format_data + "Max order amount (Limit Order)")
        PrintBasic.print_basic(self.sell_market_min_order_amt, format_data + "Min order amount (Sell Market Order)")
        PrintBasic.print_basic(self.sell_market_max_order_amt, format_data + "Max order amount (Sell Market Order)")
        PrintBasic.print_basic(self.buy_market_max_order_value, format_data + "Max order value (Buy Market Order)")
        PrintBasic.print_basic(self.max_order_value, format_data + "Max order value (In USDT)")



from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.subuser import *
from notecoin.huobi.model.subuser import TradeMarket
from notecoin.huobi.utils import *
from notecoin.huobi.utils.json_parser import default_parse_data_as_long


class GetUidService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/user/uid"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetUserApikeyInfoService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/user/api-key"

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), UserApikeyInfo)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class PostSubuserCreationService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/creation"

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), SubuserCreation)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSetSubuserTransferability:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/transferability"

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), SubuserTransferability)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSubuserApikeyDeletionService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/api-key-deletion"

        # {'code': 200, 'data': None, 'ok': True}
        def parse(dict_data):
            return dict_data

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSubuserApikeyGenerationService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/api-key-generation"

        def parse(dict_data):
            return default_parse(dict_data.get("data", {}), SubuserApikeyGeneration)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostSubuserApikeyModificationService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/api-key-modification"

        def parse(dict_data):
            return default_parse(dict_data.get("data", {}), SubuserApikeyModification)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostTradableMarketService:
    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/tradable-market"

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), TradeMarket)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from notecoin.huobi.constant import *
from notecoin.huobi.model.account import Balance
from notecoin.huobi.utils import *
from notecoin.huobi.utils import default_parse_list_dict


class LoanInfo:
    """
    The margin rate define.

    :member
        currency: The currency name.
        interest_rate: all interest rate
        min_loan_amt: min loan amount.
        max_loan_amt: max loan amount.
        loanable_amt: loanable amount.
        actual_rate: rate after deduction.
    """

    def __init__(self):
        self.currency = ""
        self.interest_rate = ""
        self.min_loan_amt = ""
        self.max_loan_amt = ""
        self.loanable_amt = ""
        self.actual_rate = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.interest_rate, format_data + "Interest Rate")
        PrintBasic.print_basic(self.min_loan_amt, format_data + "Min Loan Amount")
        PrintBasic.print_basic(self.max_loan_amt, format_data + "Max Loan Amount")
        PrintBasic.print_basic(self.loanable_amt, format_data + "Loanable Amount")
        PrintBasic.print_basic(self.actual_rate, format_data + "Actual Rate")


class LoanOrder:
    """
    The margin order information.

    :member
        id: The order id.
        user_id: The user id.
        account_type: The account type which created the loan order.
        currency: The currency name.
        loan_amount: The amount of the origin loan.
        loan_balance: The amount of the loan left.
        interest_rate: The loan interest rate.
        interest_amount: The accumulated loan interest.
        interest_balance: The amount of loan interest left.
        state: The loan stats, possible values: created, accrual, cleared, invalid.
        created_at: The UNIX formatted timestamp in UTC when the order was created.
        accrued_at: The UNIX formatted timestamp in UTC when the last accrue happened.
    """

    def __init__(self):
        self.currency = ""
        self.deduct_rate = 0
        self.paid_point = 0.0
        self.deduct_currency = ""
        self.user_id = 0
        self.created_at = 0
        self.account_id = 0
        self.paid_coin = 0.0
        self.loan_amount = 0.0
        self.interest_amount = 0.0
        self.deduct_amount = 0.0
        self.loan_balance = 0.0
        self.interest_balance = 0.0
        self.updated_at = 0
        self.accrued_at = 0
        self.interest_rate = 0.0
        self.id = 0
        self.state = LoanOrderState.INVALID

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.deduct_rate, format_data + "Deduct Rate")
        PrintBasic.print_basic(self.paid_point, format_data + "Paid Point")
        PrintBasic.print_basic(self.deduct_currency, format_data + "Deduct Currency")
        PrintBasic.print_basic(self.user_id, format_data + "User Id")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.paid_coin, format_data + "Paid Coin")
        PrintBasic.print_basic(self.loan_amount, format_data + "Load Amount")
        PrintBasic.print_basic(self.interest_amount, format_data + "Interest Amount")
        PrintBasic.print_basic(self.deduct_amount, format_data + "Deduct Amount")
        PrintBasic.print_basic(self.loan_balance, format_data + "Loan Balance")
        PrintBasic.print_basic(self.interest_balance, format_data + "Interest Balance")
        PrintBasic.print_basic(self.updated_at, format_data + "Update Time")
        PrintBasic.print_basic(self.accrued_at, format_data + "Accrued Time")
        PrintBasic.print_basic(self.interest_rate, format_data + "Interest Rate")
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.state, format_data + "Loan Order State")


class MarginAccountBalance:
    """
    The margin order information.

    :member
        id: Inner id.
        type: The account type.
        state: The account state.
        symbol: The symbol, like "btcusdt".
        fl_price: The trigger price.
        fl_type: The trigger type.
        risk_rate: The risk rate.
        list:Balance Object list
    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.symbol = ""
        self.fl_price = 0.0
        self.fl_type = 0.0
        self.risk_rate = 0.0
        self.list = []

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.fl_price, format_data + "Trigger Price")
        PrintBasic.print_basic(self.fl_type, format_data + "Trigger Type")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")
        if self.list and len(self.list):
            for balance_obj in self.list:
                balance_obj.print_object("\t")
                print()


class MarginLoanInfo:
    """
    The margin loan info.

    :member
        symbol: symbol like "btcusdt"
        currencies: loan info for currency in symbol
    """

    def __init__(self):
        self.symbol = ""
        self.currencies = list()

    @staticmethod
    def json_parse(json_data):
        retList = []
        for idx, item in enumerate(json_data):
            margin_loan_obj = MarginLoanInfo()
            margin_loan_obj.symbol = item.get("symbol", "")

            currencies_json = item.get("currencies")
            result_list = default_parse_list_dict(currencies_json, LoanInfo, [])

            margin_loan_obj.currencies = result_list

            retList.append(margin_loan_obj)

        return retList

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        if self.currencies and len(self.currencies):
            for currency_item in self.currencies:
                currency_item.print_object("\t")
                print()


class CrossMarginAccountBalance:

    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        type: The type of this account, possible value: spot, margin, otc, point.
        state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class
    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.risk_rate = 0
        self.acct_balance_sum = 0.0
        self.debt_balance_sum = 0.0
        self.list = list()

    @staticmethod
    def json_parse(data_json):
        balance_list_json = data_json.get("list", [])
        data_json.pop("list")

        account_balance = default_parse_list_dict(data_json, CrossMarginAccountBalance)
        account_balance.list = default_parse_list_dict(balance_list_json, Balance, [])

        return account_balance

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Account ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")
        PrintBasic.print_basic(self.acct_balance_sum, format_data + "Total Balance")
        PrintBasic.print_basic(self.debt_balance_sum, format_data + "Debt Balance")
        if self.list and len(self.list):
            for balance in self.list:
                balance.print_object("\t")
                print()


class CrossMarginLoanInfo(LoanInfo):
    def __init__(self):
        LoanInfo.__init__(self)

    def print_object(self, format_data=""):
        LoanInfo.print_object(self)


class GeneralRepayLoanRecord:
    """
    The general repay loan record information.

    :member
        repayId: repayment transaction ID.
        repayTime: repayment transaction time (unix time in millisecond).
        accountId: repayment account ID.
        currency: repayment currency, like "usdt".
        repaidAmount: repaid amount.
        transactIds: ID list of original loan transactions (arranged by order of repayment time).
        nextId: search the start ID in the next page (return only when there is data in the next page).
    """

    def __init__(self):
        self.repayId = None
        self.repayTime = None
        self.accountId = None
        self.currency = None
        self.repaidAmount = None
        self.transactIds = Transact()
        self.nextId = None

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.repayId, format_data + "repayId")
        PrintBasic.print_basic(self.repayTime, format_data + "repayTime")
        PrintBasic.print_basic(self.accountId, format_data + "accountId")
        PrintBasic.print_basic(self.currency, format_data + "currency")
        PrintBasic.print_basic(self.repaidAmount, format_data + "repaidAmount")
        PrintBasic.print_basic(self.transactIds, format_data + "transactIds")
        PrintBasic.print_basic(self.nextId, format_data + "nextId")

        print()


class Transact:

    """
    The general repay loan record information.

    :member
        transactId: original loan transaction ID.
        repaidPrincipal: principal repaid.
        repaidInterest: interest repaid.
        paidHt: HT paid.
        paidPoint: point paid.
    """

    def __init__(self):
        self.transactId = None
        self.repaidPrincipal = None
        self.repaidInterest = None
        self.paidHt = None
        self.paidPoint = None


class GeneralRepayLoanResult:
    """
    The margin order information.

    :member
        id: Inner id.
        type: The account type.
        state: The account state.
        symbol: The symbol, like "btcusdt".
        fl_price: The trigger price.
        fl_type: The trigger type.
        risk_rate: The risk rate.
        list:Balance Object list
    """

    def __init__(self):
        self.repayId = 0
        self.repayTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.repayId, format_data + "repayId")
        PrintBasic.print_basic(self.repayTime, format_data + "repayTime")

        print()



from notecoin.huobi.utils.input_checker import *
from notecoin.huobi.model.wallet import *


class WalletClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def get_deposit_withdraw(self, op_type: 'str', currency: 'str' = None, from_id: 'int' = None, size: 'int' = None,
                             direct: 'str' = None) -> list:
        """
        Get the withdraw records of an account.

        :param currency: The currency, like "btc". (optional)
        :param from_id: The beginning withdraw record id. (optional)
        :param op_type: deposit or withdraw, see defination DepositWithdraw (mandatory)
        :param size: The size of record. (optional)
        :param direct: "prev" is order by asc, "next" is order by desc, default as "prev"(optional)
        :return: The list of withdraw records.
        """
        check_should_not_none(op_type, "operate type")

        params = {
            "currency": currency,
            "type": op_type,
            "from": from_id,
            "direct": direct,
            "size": size
        }

        from huobi.service.wallet.get_deposit_withdraw import GetDepositWithdrawService
        return GetDepositWithdrawService(params).request(**self.__kwargs)

    def post_create_withdraw(self, address: 'str', amount: 'float', currency: 'str', fee: 'float',
                             chain: 'str' = None, address_tag: 'str' = None) -> int:
        """
        Submit a request to withdraw some asset from an account.

        :param address: The destination address of this withdraw. (mandatory)
        :param amount: The amount of currency to withdraw. (mandatory)
        :param currency: The crypto currency to withdraw. (mandatory)
        :param fee: The fee to pay with this withdraw. (mandatory)
        :param address_tag: A tag specified for this address. (optional)
        :param chain: set as "usdt" to withdraw USDT to OMNI, set as "trc20usdt" to withdraw USDT to TRX. (optional)
        :return: Withdraw id
        """
        check_symbol(currency)
        check_should_not_none(address, "address")
        check_should_not_none(amount, "amount")
        check_should_not_none(fee, "fee")

        params = {
            "currency": currency,
            "address": address,
            "amount": amount,
            "fee": fee,
            "chain": chain,
            "addr-tag": address_tag
        }

        from huobi.service.wallet.post_create_withdraw import PostCreateWithdrawService
        return PostCreateWithdrawService(params).request(**self.__kwargs)

    def post_cancel_withdraw(self, withdraw_id: 'int') -> int:
        """
        Cancel an withdraw request.

        :param withdraw_id: withdraw id (mandatory)
        :return: No return.
        """
        params = {
            "withdraw-id": withdraw_id
        }

        from huobi.service.wallet.post_cancel_withdraw import PostCancelWithdrawService
        return PostCancelWithdrawService(params).request(**self.__kwargs)

    def get_account_deposit_address(self, currency: 'str') -> list:
        """
        Get deposit address of corresponding chain, for a specific crypto currency (except IOTA)

        :param currency: The currency, like "btc". (optional)
        :return:
        """
        check_should_not_none(currency, "currency")

        params = {
            "currency": currency
        }

        from huobi.service.wallet.get_account_deposit_address import GetAccountDepositAddressService
        return GetAccountDepositAddressService(params).request(**self.__kwargs)

    def get_account_withdraw_quota(self, currency: 'str') -> list:
        """
        Get the withdraw quota for currencies

        :param currency: The currency, like "btc". (mandatory)
        :return:
        """
        check_should_not_none(currency, "currency")

        params = {
            "currency": currency,
        }

        from huobi.service.wallet.get_account_withdraw_quota import GetAccountWithdrawQuotaService
        return GetAccountWithdrawQuotaService(params).request(**self.__kwargs)

    def get_sub_user_deposit_history(self, sub_uid: 'int', currency: 'str' = None,
                                     start_time: 'int' = None, end_time: 'int' = None,
                                     sort: 'str' = None, limit: 'int' = None, from_id: 'int' = None) -> DepositHistory:
        """
        Parent get sub user depoist history.

        :param sub_uid: Sub user id. (mandatory)
        :param currency: Cryptocurrency.
        :param start_time: Farthest time
        :param end_time: Nearest time
        :param sort: Sorting order
        :param limit: Maximum number of items in one page
        :param from_id: First record Id in this query
        """
        check_should_not_none(sub_uid, "sub_uid")

        params = {
            "subUid": sub_uid,
            "currency": currency,
            "startTime": start_time,
            "endTime": end_time,
            "sort": sort,
            "limit": limit,
            "fromId": from_id
        }

        from huobi.service.wallet.get_sub_user_deposit_history import GetSubUserDepositHistoryService
        return GetSubUserDepositHistoryService(params).request(**self.__kwargs)

    def get_sub_user_deposit_address(self, sub_uid: 'int', currency: 'str') -> list:
        """
        Parent get sub user deposit address

        :param sub_uid: Sub user id
        :param currency: Cryptocurrency, like "btc". (mandatory)
        :return:
        """

        check_should_not_none(sub_uid, "subUid")
        check_should_not_none(currency, "currency")
        params = {
            "subUid": sub_uid,
            "currency": currency
        }

        from huobi.service.wallet.get_sub_user_deposit_address import GetSubUserDepositAddressService
        return GetSubUserDepositAddressService(params).request(**self.__kwargs)

    def get_account_withdraw_address(self, currency: 'str', chain: 'str'=None, note: 'str'=None, limit: 'int' = 100,
                                     fromid: 'int' = None):
        check_should_not_none(currency, "currency")
        params = {
            "currency": currency,
            "chain": chain,
            "note": note,
            "limit": limit,
            "fromid": fromid
        }
        from huobi.service.wallet.get_account_withdraw_address import GetAccountWithdrawAddressService
        return GetAccountWithdrawAddressService(params).request(**self.__kwargs)




class SubuserApikeyGeneration:
    """
    The trade information with price and amount etc.

    :member
        accessKey:
        secretKey:
        note:
        permission: "trade,readOnly",
        ipAddresses":
    """

    def __init__(self):
        self.accessKey = ""
        self.secretKey = ""
        self.note = ""
        self.permission = ""
        self.ipAddresses = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accessKey, format_data + "accessKey")
        PrintBasic.print_basic(self.secretKey, format_data + "secretKey")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.permission, format_data + "permission")
        PrintBasic.print_basic(self.ipAddresses, format_data + "ipAddresses")


class SubuserApikeyModification:
    """
    The trade information with price and amount etc.

    :member
        note:
        permission: "trade,readOnly",
        ipAddresses":
    """

    def __init__(self):
        self.note = ""
        self.permission = ""
        self.ipAddresses = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.permission, format_data + "permission")
        PrintBasic.print_basic(self.ipAddresses, format_data + "ipAddresses")


class SubuserCreation:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        userState: sub user account state, states see SubUidState.
    """

    def __init__(self):
        self.user_name = ""
        self.note = ""
        self.uid = 0
        self.err_code = 0
        self.err_message = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.user_name, format_data + "userName")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.uid, format_data + "uid")
        PrintBasic.print_basic(self.err_code, format_data + "errCode")
        PrintBasic.print_basic(self.err_message, format_data + "errMessage")


class SubuserTransferability:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        userState: sub user account state, states see SubUidState.
    """

    def __init__(self):
        self.transferrable = ""
        self.accountType = ""
        self.subUid = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.subUid, format_data + "subUid")
        PrintBasic.print_basic(self.accountType, format_data + "accountType")
        PrintBasic.print_basic(self.transferrable, format_data + "transferrable")


class TradeMarket:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        accountType:
        activation: sub user account state for given accountType.
    """

    def __init__(self):
        self.sub_uid = ""
        self.account_type = ""
        self.activation = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.sub_uid, format_data + "subUid")
        PrintBasic.print_basic(self.account_type, format_data + "accountType")
        PrintBasic.print_basic(self.activation, format_data + "activation")


class UserApikeyInfo:
    """
    The trade information with price and amount etc.

    :member
        accessKey: .
        createTime:
        ipAddresses: .
        note:
        permission:
        status:
        updateTime:
        validDays:

    """

    def __init__(self):
        self.accessKey = ""
        self.createTime = 0
        self.ipAddresses = ""
        self.note = ""
        self.permission = ""
        self.status = ""
        self.updateTime = 0
        self.validDays = -1

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic

        PrintBasic.print_basic(self.accessKey, format_data + "accessKey")
        PrintBasic.print_basic(self.createTime, format_data + "createTime")
        PrintBasic.print_basic(self.ipAddresses, format_data + "ipAddresses")
        PrintBasic.print_basic(self.note, format_data + "note")
        PrintBasic.print_basic(self.permission, format_data + "permission")
        PrintBasic.print_basic(self.status, format_data + "status")
        PrintBasic.print_basic(self.updateTime, format_data + "updateTime")
        PrintBasic.print_basic(self.validDays, format_data + "validDays")



import pandas as pd


class Response:
    def __init__(self, dict_data, code=None, data=None):
        self.code = dict_data.get('code', -1)
        self.data = dict_data.get('data', [])
        self.dict_data = dict_data
        if self.data is not None and len(self.data)>0:
            self.data=pd.DataFrame(self.data)

    def __str__(self):
        return "{} {}".format(self.code, len(self.data))



from notecoin.huobi.constant import SubUidState
from notecoin.huobi.constant import *
from notecoin.huobi.utils import default_parse, default_parse_list_dict


class AccountAssetValuationResult:
    """
    The account information for spot account, margin account etc.

    :member
        balance: balance valuation bases on given valuation currency.
        timestamp: unix timestamp from server.

    """

    def __init__(self):
        self.balance = ""
        self.timestamp = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.balance, format_data + "balance")
        PrintBasic.print_basic(self.timestamp, format_data + "timestamp")


class AccountBalanceReq:
    """
    The account change information received by subscription of account.

    :member
        ts: The UNIX formatted timestamp generated by server in UTC.
        cid: client request ID
        topic: request Channel or Topic
        data: The list of account and balance

    """

    def __init__(self):
        self.ts = 0
        self.cid = ""
        self.topic = ""
        self.data = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        PrintBasic.print_basic(self.cid, format_data + "Client Order ID")
        PrintBasic.print_basic(self.topic, format_data + "Topic")
        print()
        if len(self.data):
            for account_balance in self.data:
                account_balance.print_object()
                print()


class AccountBalance:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.subtype = ""
        self.list = list()

    @staticmethod
    def json_parse(data_dict):
        if data_dict and len(data_dict):
            balance_list = data_dict.get("list")
            data_dict.pop("list")
            account_balance_obj = default_parse(data_dict, AccountBalance, Balance)
            account_balance_obj.subtype = data_dict.get("subtype", data_dict.get("symbol"))
            account_balance_obj.list = default_parse_list_dict(balance_list, Balance, [])
            return account_balance_obj

        return None

    @staticmethod
    def json_parse_list(data_list):
        account_balance_list = []
        if data_list and len(data_list):
            for item in data_list:
                item_obj = AccountBalance.json_parse(item)
                if item_obj:
                    account_balance_list.append(item_obj)
        return account_balance_list

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Account ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.subtype, format_data + "Subtype")

        print()
        if len(self.list):
            for row in self.list:
                row.print_object(format_data+"\t")
                print()


class AccountHistory:
    """
    The account information for spot account, margin account etc.

    :member
        account_id: Account Id.
        currency: Currency name
        transact_amt: Amount change (positive value if income, negative value if outcome)
        transact-type: Amount change type
        avail_balance: Available balance
        acct_balance: Account balance
        transact_time: Transaction time (database time)
        record_id: Unique record ID in the database

    """

    def __init__(self):
        self.account_id = 0
        self.currency = ""
        self.transact_amt = ""
        self.transact_type = ""
        self.avail_balance = ""
        self.acct_balance = ""
        self.transact_time = 0
        self.record_id = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.transact_amt, format_data + "Transact Amount")
        PrintBasic.print_basic(self.transact_type, format_data + "Transact Type")
        PrintBasic.print_basic(self.avail_balance, format_data + "Avail Balance")
        PrintBasic.print_basic(self.acct_balance, format_data + "Account Balance")
        PrintBasic.print_basic(self.transact_time, format_data + "Transact Time")
        PrintBasic.print_basic(self.record_id, format_data + "Record Id")


class AccountLedger:
    """
    The account ledger information.

    :member
        accountId: Account ID.
        currency: Cryptocurrency.
        transactAmt: Transaction amount (income positive, expenditure negative).
        transactType: Transaction type.
        transferType: Transfer type (only valid for transactType=transfer).
        transactId: Transaction ID.
        transactTime: Transaction time.
        transferer: Transferer’s account ID.
        transferee: Transferee’s account ID.
    """

    def __init__(self):
        self.accountId = 0
        self.currency = ""
        self.transactAmt = 0.0
        self.transactType = ""
        self.transferType = ""
        self.transactId = 0
        self.transactTime = 0
        self.transferer = 0
        self.transferee = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.transactAmt, format_data + "Transaction Amount")
        PrintBasic.print_basic(self.transactType, format_data + "Transaction Type")
        PrintBasic.print_basic(self.transferType, format_data + "Transfer Type")
        PrintBasic.print_basic(self.transactId, format_data + "Transaction ID")
        PrintBasic.print_basic(self.transactTime, format_data + "Transaction Time")
        PrintBasic.print_basic(self.transferer, format_data + "Transferer’s Account ID")
        PrintBasic.print_basic(self.transferee, format_data + "Transferee’s Account ID")


class AccountPointGroup:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.groupId = ""
        self.expiryDate = ""
        self.remainAmt = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.groupId, format_data + "Group Id")
        PrintBasic.print_basic(self.expiryDate, format_data + "Expiration date")
        PrintBasic.print_basic(self.remainAmt, format_data + "Remain Amount")


class AccountPointResult:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        list: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.accountId = ""
        self.accountStatus = AccountPointState.INVALID
        self.groupIds = list()
        self.acctBalance = ""

    @staticmethod
    def json_parse(data_dict):
        if data_dict and len(data_dict):
            group_ids = data_dict.get("groupIds")
            data_dict.pop("groupIds")
            account_point_obj = default_parse(data_dict, AccountPointResult, AccountPointGroup)
            account_point_obj.subtype = data_dict.get("subtype", data_dict.get("symbol"))
            account_point_obj.list = default_parse_list_dict(group_ids, AccountPointGroup, [])
            return account_point_obj

        return None

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.accountStatus, format_data + "Account Status")
        PrintBasic.print_basic(self.acctBalance, format_data + "Account Balance")

        print()
        if len(self.groupIds):
            for row in self.groupIds:
                row.print_object(format_data + "\t")
                print()


class AccountPointTransferResult:
    """
    The account change information received by subscription of account.

    :member
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        change_type: The event that asset change notification related.
        account_change_list: The list of account change, the content is AccountChange class

    """

    def __init__(self):
        self.transactId = ""
        self.transactTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.transactId, format_data + "transactId")
        PrintBasic.print_basic(self.transactTime, format_data + "transactTime")
        self.data.print_object()


class AccountTransferResult:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        balances: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.transact_id = 0
        self.transact_time = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.transact_id, format_data + "TransactionId")
        PrintBasic.print_basic(self.transact_time, format_data + "TransactionTime")


class AccountUpdateEvent:
    """
    The account change information received by subscription of account.

    :member
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        change_type: The event that asset change notification related.
        account_change_list: The list of account change, the content is AccountChange class

    """

    def __init__(self):
        self.ch = 0
        self.data = AccountUpdate()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Topic")
        self.data.print_object()


class AccountUpdate:
    """
    The account change information received by subscription of account.

    :member
        currency: The currency of the change.
        accountId: The account id.
        balance: Account balance (only exists when account balance changed)
        available:	Available balance (only exists when available balance changed)
        changeType:	Change type see AccountChangeType, valid value: order-place,order-match,order-refund,order-cancel,order-fee-refund,margin-transfer,margin-loan,margin-interest,margin-repay,other,
        accountType: Account type see AccountBalanceUpdateType, valid value: trade, frozen, loan, interest
        changeTime:	Change time, unix time in millisecond
    """

    def __init__(self):
        self.currency = ""
        self.accountId = 0
        self.balance = ""
        self.available = ""
        self.changeType = AccountChangeType.INVALID
        self.accountType = AccountBalanceUpdateType.INVALID
        self.changeTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.accountId, format_data + "Account ID")
        PrintBasic.print_basic(self.balance, format_data + "Balance")
        PrintBasic.print_basic(self.available, format_data + "Available")
        PrintBasic.print_basic(self.changeType, format_data + "Account Change Type")
        PrintBasic.print_basic(self.accountType, format_data + "Account Balance Change Type")
        PrintBasic.print_basic(self.changeTime, format_data + "Account Timestamp")


class Account:
    """
    The account information for spot account, margin account etc.

    :member
        id: The unique account id.
        account_type: The type of this account, possible value: spot, margin, otc, point.
        account_state: The account state, possible value: working, lock.
        balances: The balance list of the specified currency. The content is Balance class

    """

    def __init__(self):
        self.id = 0
        self.type = AccountType.INVALID
        self.state = AccountState.INVALID
        self.subtype = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.subtype, format_data + "Subtype")


class Balance:
    """
    The balance of specified account.

    :member
        currency: The currency of this balance.
        balance_type: The balance type, trade or frozen.
        balance: The balance in the main currency unit.

    """

    def __init__(self):
        self.currency = ""
        self.type = AccountBalanceUpdateType.INVALID
        self.balance = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Balance Type")
        PrintBasic.print_basic(self.balance, format_data + "Balance")


class CompleteSubAccount:
    """
    Sub-account completed info

    :member
        id: The sub-id.
        account_type: The sub account type.
        balances: The balance list, the content is Balance class.
    """

    def __init__(self):
        self.id = 0
        self.account_type = AccountType.INVALID
        self.balances = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.account_type, format_data + "Account Type")
        if len(self.balances):
            for row in self.balances:
                row.print_object()
                print()


class MarginBalanceDetail:
    def __init__(self):
        self.id = 0
        self.symbol = 0
        self.state = AccountState.INVALID
        self.type = AccountType.INVALID
        self.risk_rate = 0.0
        self.fl_price = 0.0
        self.fl_type = ""
        self.sub_account_balance_list = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.fl_price, format_data + "Burst Price")
        PrintBasic.print_basic(self.fl_type, format_data + "Burst Type")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")


class SubUidManagement:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        userState: sub user account state, states see SubUidState.
    """

    def __init__(self):
        self.subUid = 0
        self.userState = SubUidState.INVALID

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.subUid, format_data + "subUid")
        PrintBasic.print_basic(self.userState, format_data + "userState")



from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.constant import *
from notecoin.huobi.model.trade import *
from notecoin.huobi.model.wallet import *
from notecoin.huobi.utils import *
from notecoin.huobi.utils.json_parser import default_parse_data_as_long


class GetAccountDepositAddressService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/account/deposit/address"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, ChainDepositAddress)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountWithdrawAddressService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/account/withdraw/address"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, ChainWithdrawAddress)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetAccountWithdrawQuotaService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/account/withdraw/quota"

        def parse(dict_data):
            data = dict_data.get("data", {})
            chains = data.get("chains", [])
            return default_parse_list_dict(chains, WithdrawQuota)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetDepositWithdrawService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/query/deposit-withdraw"

        def parse(dict_data):
            op_type = self.params["type"]
            data_list = dict_data.get("data", [])
            if op_type == DepositWithdraw.DEPOSIT:
                return default_parse_list_dict(data_list, Deposit)
            elif op_type == DepositWithdraw.WITHDRAW:
                return default_parse_list_dict(data_list, Withdraw)
            return []

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetSubUserDepositAddressService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/deposit-address"

        def parse(dict_data):
            json_data_list = dict_data.get("data", [])
            return default_parse_list_dict(json_data_list, ChainDepositAddress, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class GetSubUserDepositHistoryService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/query-deposit"

        def parse(dict_data):
            deposit_history = DepositHistory()
            deposit_history.nextId = dict_data.get("nextId", 0)
            json_data_list = dict_data.get("data", [])
            deposit_history_item_list = default_parse_list_dict(json_data_list, DepositHistoryItem)
            deposit_history.data = deposit_history_item_list
            return deposit_history

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class PostCancelWithdrawService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        withdraw_id_params = self.params["withdraw-id"]

        def get_channel():
            path = "/v1/dw/withdraw-virtual/{}/cancel"
            return path.format(withdraw_id_params)

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, get_channel(), self.params, parse)


class PostCreateWithdrawService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/dw/withdraw/api/create"

        def parse(dict_data):
            # return dict_data.get("data", 0)
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



import re
import time
from notecoin.huobi.exception.huobi_api_exception import HuobiApiException
from notecoin.huobi.constant.definition import *

reg_ex = "[ _`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]|\n|\t"


def check_symbol(symbol):
    if not isinstance(symbol, str):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] symbol must be string")
    if re.match(reg_ex, symbol):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + symbol + "  is invalid symbol")


def check_time_in_force(time_in_force, order_type):
    if time_in_force is None:
        return

    if order_type in [OrderType.BUY_MARKET, OrderType.SELL_MARKET] \
            and time_in_force in [TimeInForceType.GTC, TimeInForceType.BOC, TimeInForceType.FOK]:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] timeInForce not supported for market order")


def check_symbol_list(symbols):
    if not isinstance(symbols, list):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] symbols in subscription is not a list")
    for symbol in symbols:
        check_symbol(symbol)


def check_currency(currency):
    if not isinstance(currency, str):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] currency must be string")
    if re.match(reg_ex, currency) is not None:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + currency + "  is invalid currency")


def check_range(value, min_value, max_value, name):
    if value is None:
        return
    if min_value > value or value > max_value:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR,
                                "[Input] " + name + " is out of bound. " + str(value) + " is not in [" + str(
                                    min_value) + "," + str(max_value) + "]")


def check_should_not_none(value, name):
    if value is None:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + name + " should not be null")


def check_should_none(value, name):
    if value is not None:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + name + " should be null")


def check_in_list(value, list_configed, name):
    if (value is not None) and (value not in list_configed):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR,
                                "[Input] " + name + " should be one in " + (",".join(list_configed)))


def check_list(list_value, min_value, max_value, name):
    if list_value is None:
        return
    if len(list_value) > max_value:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR,
                                "[Input] " + name + " is out of bound, the max size is " + str(max_value))
    if len(list_value) < min_value:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR,
                                "[Input] " + name + " should contain " + str(min_value) + " item(s) at least")


def greater_or_equal(value, base, name):
    if value is not None and value < base:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR,
                                "[Input] " + name + " should be greater than " + base)


def format_date(value, name):
    if value is None:
        return None
    if not isinstance(value, str):
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + name + " must be string")
    try:
        new_time = time.strptime(value, "%Y-%m-%d")
        return time.strftime("%Y-%m-%d", new_time)
    except:
        raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + name + " is not invalid date format")



from notecoin.huobi.connection.restapi_sync_client import RestApiSyncClient
from notecoin.huobi.constant.system import HttpMethod
from notecoin.huobi.model.etf import *
from notecoin.huobi.utils import *


class GetEtfSwapConfigService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/etf/swap/config"

        def parse(dict_data):
            data_info = dict_data.get("data", {})
            return default_parse(data_info, EtfSwapConfig, UnitPrice)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)


class GetEtfSwapListService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/etf/swap/list"

        def parse(dict_data):
            return EtfSwapList.json_parse_list(dict_data.get("data", []))

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)


class PostEftSwapInService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/etf/swap/in"

        def parse(dict_data):
            return default_parse_fill_directly(dict_data, EtfSwapInOut)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)


class PostEtfSwapOutService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/etf/swap/out"

        def parse(dict_data):
            return default_parse_fill_directly(dict_data, EtfSwapInOut)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from notecoin.huobi.constant import *
from notecoin.huobi.model.etf import UnitPrice
from notecoin.huobi.utils import default_parse_list_dict


class EtfSwapConfig:
    """
    The basic information of ETF creation and redemption, as well as ETF constituents, including max
    amount of creation, min amount of creation, max amount of redemption, min amount of redemption,
    creation fee rate, redemption fee rate, eft create/redeem status.

    :member
        purchase_max_amount: The max creation amounts per request.
        purchase_min_amount: The minimum creation amounts per request.
        redemption_max_amount: The max redemption amounts per request.
        redemption_min_amount: The minimum redemption amounts per request.
        purchase_fee_rate: The creation fee rate.
        redemption_fee_rate: The redemption fee rate.
        status: The status of the ETF.
        unit_price_list: ETF constitution in format of amount and currency.

    """

    def __init__(self):
        self.etf_name = ""
        self.etf_status = EtfStatus.INVALID
        self.purchase_fee_rate = 0.0
        self.purchase_max_amount = 0
        self.purchase_min_amount = 0
        self.redemption_fee_rate = 0.0
        self.redemption_max_amount = 0
        self.redemption_min_amount = 0
        self.unit_price = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.etf_name, format_data + "ETF Name")
        PrintBasic.print_basic(self.etf_status, format_data + "ETF Status")

        PrintBasic.print_basic(self.purchase_fee_rate, format_data + "Purchase Fee Rate")
        PrintBasic.print_basic(self.purchase_max_amount, format_data + "Purchase Max Amount")
        PrintBasic.print_basic(self.purchase_min_amount, format_data + "Purchase Min Amount")

        PrintBasic.print_basic(self.redemption_fee_rate, format_data + "Redemption Fee Rate")
        PrintBasic.print_basic(self.redemption_max_amount, format_data + "Redemption Max Amount")
        PrintBasic.print_basic(self.redemption_min_amount, format_data + "Redemption Min Amount")
        print()
        if len(self.unit_price):
            for row in self.unit_price:
                row.print_object(format_data)
                print()


class EtfSwapInOut:
    """
    :member
    """

    def __init__(self):
        self.code = 0
        self.data = None
        self.message = ""
        self.success = False

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.code, format_data + "Return Code")
        PrintBasic.print_basic_bool(self.data, format_data + "Data")
        PrintBasic.print_basic(self.message, format_data + "Message")
        PrintBasic.print_basic_bool(self.success, format_data + "Success")


class EtfSwapList:
    """
    The past creation and redemption.

    :member
        id: the operation Id.
        gmt_created: The UNIX formatted timestamp in UTC of the operation.
        currency: The ETF name.
        amount: Creation or redemption amount.
        type: The swap type. Creation or redemption.
        status: The operation result
        rate: The fee rate.
        fee: The actual fee amount

        point_card_amount: Discount from point card.
        used_currency_list: For creation this is the list and amount of underlying assets used for ETF creation.
            For redemption this is the amount of ETF used for redemption. The content is UnitPrice class.
        obtain_currency_list: For creation this is the amount for ETF created.
            For redemption this is the list and amount of underlying assets obtained. The content is UnitPrice class
    """

    def __init__(self):
        self.id = 0
        self.gmt_created = 0
        self.currency = ""
        self.amount = 0.0
        self.type = EtfSwapType.INVALID
        self.status = 0
        self.rate = 0.0
        self.fee = 0.0
        self.point_card_amount = 0.0
        self.used_currency_list = list()
        self.obtain_currency_list = list()

    @staticmethod
    def json_parse(dict_data):
        if dict_data and len(dict_data):
            detail = dict_data.get("detail", {})
            dict_data.pop("detail")
            etf_swap_obj = default_parse_list_dict(dict_data, EtfSwapList)
            if detail and len(detail):
                etf_swap_obj.rate = detail.get("rate", 0)
                etf_swap_obj.fee = detail.get("fee", 0)
                etf_swap_obj.point_card_amount = detail.get("point_card_amount", 0)
                etf_swap_obj.used_currency_list = default_parse_list_dict(
                    detail.get("used_currency_list"), UnitPrice, [])
                etf_swap_obj.obtain_currency_list = default_parse_list_dict(
                    detail.get("obtain_currency_list"), UnitPrice, [])
            return etf_swap_obj

        return None

    @staticmethod
    def json_parse_list(dict_data):
        ret_list = list()
        for item in dict_data:
            item_obj = EtfSwapList.json_parse(item)
            if item_obj is not None:
                ret_list.append(item_obj)

        return ret_list

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Operater Id")
        PrintBasic.print_basic(self.gmt_created, format_data + "GMT Create Time")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Type")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.rate, format_data + "Rate")
        PrintBasic.print_basic(self.fee, format_data + "Fee")
        PrintBasic.print_basic(self.status, format_data + "Status")
        PrintBasic.print_basic(self.point_card_amount, format_data + "Point Card Amount")

        if len(self.used_currency_list):
            PrintBasic.print_basic("used_currency_list as below:")
            for row in self.used_currency_list:
                row.print_object(format_data + "\t")

        if len(self.obtain_currency_list):
            PrintBasic.print_basic("obtain_currency_list as below:")
            for row in self.obtain_currency_list:
                row.print_object(format_data + "\t")


class UnitPrice:

    def __init__(self):
        self.currency = ""
        self.amount = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
